@@ -322,19 +322,17 @@ def request_server(server_widget):
     dialog.show_all()
     dialog.set_default_response(gtk.RESPONSE_OK)
 
-    url_m = re.match('^([\w.-]+):(\d{1,5})',
-        server_widget.get_text())
-    if url_m:
-        entry_server.set_text(url_m.group(1))
-        entry_port.set_text(url_m.group(2))
+    netloc = server_widget.get_text()
+    entry_server.set_text(get_hostname(netloc))
+    entry_port.set_text(str(get_port(netloc)))
 
     res = dialog.run()
     if res == gtk.RESPONSE_OK:
         host = entry_server.get_text()
-        port = int(entry_port.get_text())
-        url = '%s:%d' % (host, port)
+        port = entry_port.get_text()
+        url = '%s:%s' % (host, port)
         server_widget.set_text(url)
-        result = (host, port)
+        result = (get_hostname(url), get_port(url))
     parent.present()
     dialog.destroy()
     return result
@@ -1067,7 +1065,6 @@ def process_exception(exception, *args, **kwargs):
                 'until its fingerprint is fixed.'), _('Security risk!'))
             from tryton.gui.main import Main
             Main.sig_quit()
-            sys.exit()
         elif exception.faultCode == 'NotLogged':
             if rpc.CONNECTION is None:
                 message(_('Connection error!\n'
@@ -1131,8 +1128,7 @@ def process_exception(exception, *args, **kwargs):
                 while True:
                     password = ask(_('Password:'), visibility=False)
                     if password is None:
-                        Main.get_main().sig_logout()
-                        return False
+                        Main.get_main().sig_quit()
                     res = rpc.login(rpc._USERNAME, password, hostname, port,
                             rpc._DATABASE)
                     if res == -1:
@@ -1534,3 +1530,55 @@ def humanize(size):
         if size < 1000:
             return '%3.1f%s' % (size, x)
         size /= 1000.0
+
+
+def get_hostname(netloc):
+    if '[' in netloc and ']' in netloc:
+        return netloc.split(']')[0][1:]
+    elif ':' in netloc:
+        return netloc.split(':')[0]
+    else:
+        return netloc
+
+
+def get_port(netloc):
+    netloc = netloc.split(']')[-1]
+    if ':' in netloc:
+        try:
+            return int(netloc.split(':')[1])
+        except ValueError:
+            pass
+    return 8000
+
+
+def resize_pixbuf(pixbuf, width, height):
+    img_height = pixbuf.get_height()
+    height = min(img_height, height) if height != -1 else img_height
+    img_width = pixbuf.get_width()
+    width = min(img_width, width) if width != -1 else img_width
+
+    if img_width / width < img_height / height:
+        width = float(img_width) / float(img_height) * float(height)
+    else:
+        height = float(img_height) / float(img_width) * float(width)
+    return pixbuf.scale_simple(int(width), int(height),
+        gtk.gdk.INTERP_BILINEAR)
+
+
+def _data2pixbuf(data):
+    loader = gtk.gdk.PixbufLoader()
+    loader.write(data, len(data))
+    loader.close()
+    return loader.get_pixbuf()
+
+BIG_IMAGE_SIZE = 10 ** 6
+with open(os.path.join(PIXMAPS_DIR, 'tryton-noimage.png'), 'rb') as no_image:
+    NO_IMG_PIXBUF = _data2pixbuf(no_image.read())
+
+
+def data2pixbuf(data):
+    try:
+        pixbuf = _data2pixbuf(data)
+    except glib.GError:
+        pixbuf = NO_IMG_PIXBUF
+    return pixbuf
