@@ -117,12 +117,20 @@ def quote(value):
     "Quote string if needed"
     if not isinstance(value, basestring):
         return value
+    if '"' in value:
+        value = value.replace('"', '\\"')
     for test in (':', ' ', '(', ')') + OPERATORS:
         if test in value:
             return '"%s"' % value
     return value
 
 
+def test_quote():
+    assert quote('test') == 'test'
+    assert quote('foo bar') == '"foo bar"'
+    assert quote('"foo"') == '\\\"foo\\\"'
+
+
 def ending_clause(domain, deep=0):
     "Return the ending clause"
     if not domain:
@@ -1082,7 +1090,7 @@ class DomainParser(object):
                             rvalue = convert_value(field, rvalue)
                             yield iter([
                                     (field_name, '>=', lvalue),
-                                    (field_name, '<', rvalue),
+                                    (field_name, '<=', rvalue),
                                     ])
                             continue
                     if isinstance(value, list):
@@ -1284,6 +1292,9 @@ def test_group():
         ('Name', '=', None),
         ('Name', None, 'Doe'),
         ]
+    assert rlist(dom.group(udlex(u'Name: \\"foo\\"'))) == [
+        ('Name', None, '"foo"'),
+        ]
 
 
 def test_parse_clause():
@@ -1355,7 +1366,7 @@ def test_parse_clause():
         ]
     assert rlist(dom.parse_clause([('Integer', None, '3..5')])) == [[
             ('integer', '>=', 3),
-            ('integer', '<', 5),
+            ('integer', '<=', 5),
             ]]
     assert rlist(dom.parse_clause([('Reference', None, 'foo')])) == [
         ('reference', 'ilike', '%foo%'),
