@@ -144,9 +144,6 @@ class Main(object):
         self.menuitem_user = None
         self.menuitem_favorite = None
 
-        if self.macapp is not None:
-            self.macapp.ready()
-
         self.buttons = {}
 
         self.pane = gtk.HPaned()
@@ -204,6 +201,9 @@ class Main(object):
         # Register plugins
         tryton.plugins.register()
 
+        if self.macapp is not None:
+            self.macapp.ready()
+
         _MAIN.append(self)
 
     def set_menubar(self):
@@ -358,9 +358,9 @@ class Main(object):
                 widget.emit('changed')
                 end()
 
-            RPCExecute('model', 'ir.model', 'global_search',
-                    search_text, CONFIG['client.limit'],
-                    self.menu_screen.model_name, callback=set_result)
+            RPCExecute('model', 'ir.model', 'global_search', search_text,
+                CONFIG['client.limit'], self.menu_screen.model_name,
+                context=self.menu_screen.context, callback=set_result)
             return False
 
         def changed(widget):
@@ -764,7 +764,7 @@ class Main(object):
 
     def favorite_set(self):
         if not self.menu_screen:
-            return
+            return False
 
         def _action_favorite(widget, id_):
             Action.exec_keyword('tree_open', {
@@ -801,12 +801,19 @@ class Main(object):
         menu.add(manage_favorites)
         menu.show_all()
         self.menuitem_favorite.set_submenu(menu)
+        return True
 
     def favorite_unset(self):
+        had_submenu = self.menuitem_favorite.get_submenu()
         self.menuitem_favorite.remove_submenu()
         # Set a submenu to get keyboard shortcut working
         self.menuitem_favorite.set_submenu(gtk.Menu())
 
+        if self.macapp and had_submenu:
+            # As the select event is not managed by the mac menu,
+            # it is done using a timeout
+            gobject.timeout_add(1000, lambda: not self.favorite_set())
+
     def sig_accel_change(self, value):
         CONFIG['client.can_change_accelerators'] = value
         return self.sig_accel()
@@ -1044,21 +1051,23 @@ class Main(object):
         elif action.get('view_id', False):
             view_ids = [action['view_id'][0]]
         ctx = rpc.CONTEXT.copy()
-        domain = PYSONDecoder(ctx).decode(action['pyson_domain'])
-        screen = Screen(action['res_model'], mode=['tree'],
-            view_ids=view_ids, domain=domain, readonly=True)
+        decoder = PYSONDecoder(ctx)
+        action_ctx = decoder.decode(action.get('pyson_context') or '{}')
+        domain = decoder.decode(action['pyson_domain'])
+        screen = Screen(action['res_model'], mode=['tree'], view_ids=view_ids,
+            domain=domain, context=action_ctx, readonly=True)
         # Use alternate view to not show search box
         screen.screen_container.alternate_view = True
         screen.switch_view(view_type=screen.current_view.view_type)
 
         vbox.pack_start(screen.screen_container.alternate_viewport, True, True)
-        screen.current_view.widget_tree.set_headers_visible(False)
+        treeview = screen.current_view.treeview
+        treeview.set_headers_visible(False)
 
         self.menu_expander.add(vbox)
         self.menu_expander.show()
 
         # Favorite column
-        treeview = screen.current_view.widget_tree
         column = gtk.TreeViewColumn()
         column.name = None
         column._type = None
@@ -1123,6 +1132,7 @@ class Main(object):
         gtk.accel_map_save(os.path.join(get_config_dir(), 'accel.map'))
 
         cls.tryton_client.quit_mainloop()
+        sys.exit()
 
     def sig_close(self, widget, event=None):
         if not self.sig_logout(widget):
@@ -1139,14 +1149,6 @@ class Main(object):
     def sig_window_state(self, widget, event):
         CONFIG['client.maximize'] = (event.new_window_state ==
                 gtk.gdk.WINDOW_STATE_MAXIMIZED)
-
-        if event.new_window_state == gtk.gdk.WINDOW_STATE_ICONIFIED:
-            meth = 'iconify'
-        else:
-            meth = 'deiconify'
-        for toplevel in gtk.window_list_toplevels():
-            getattr(toplevel, meth)()
-
         return False
 
     def win_add(self, page, hide_current=False, allow_similar=True):
