@@ -1,38 +1,37 @@
 #This file is part of Tryton.  The COPYRIGHT file at the top level of
 #this repository contains the full copyright notices and license terms.
-from __future__ import with_statement
 
 import ConfigParser
 import gtk
 import gobject
 import os
-import re
 import gettext
-import threading
-import time
 
 from tryton.version import VERSION
 import tryton.common as common
 from tryton.config import CONFIG, TRYTON_ICON, PIXMAPS_DIR, get_config_dir
 import tryton.rpc as rpc
 from tryton.gui.window.dbcreate import DBCreate
+from tryton.exceptions import TrytonError
 
 _ = gettext.gettext
 
 
 class DBListEditor(object):
 
-    def __init__(self, parent, profile_store, profiles):
+    def __init__(self, parent, profile_store, profiles, callback):
         self.profiles = profiles
         self.current_database = None
         self.old_profile, self.current_profile = None, None
+        self.db_cache = None
         self.updating_db = False
 
         # GTK Stuffs
         self.parent = parent
         self.dialog = gtk.Dialog(title=_(u'Profile Editor'), parent=parent,
-            flags=gtk.DIALOG_MODAL|gtk.DIALOG_DESTROY_WITH_PARENT)
-        self.ok_button = self.dialog.add_button(gtk.STOCK_OK, gtk.RESPONSE_ACCEPT)
+            flags=gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT)
+        self.ok_button = self.dialog.add_button(gtk.STOCK_OK,
+            gtk.RESPONSE_ACCEPT)
         self.dialog.set_position(gtk.WIN_POS_CENTER_ON_PARENT)
         self.dialog.set_has_separator(True)
         self.dialog.set_icon(TRYTON_ICON)
@@ -41,9 +40,7 @@ class DBListEditor(object):
         vbox_profiles = gtk.VBox(homogeneous=False, spacing=6)
         self.cell = gtk.CellRendererText()
         self.cell.set_property('editable', True)
-        self.cell.connect('edited', self.edit_profilename)
         self.cell.connect('editing-started', self.edit_started)
-        self.cell.connect('editing-canceled', self.edit_canceled)
         self.profile_tree = gtk.TreeView()
         self.profile_tree.set_model(profile_store)
         self.profile_tree.insert_column_with_attributes(-1, _(u'Profile'),
@@ -94,7 +91,8 @@ class DBListEditor(object):
         database.set_padding(3, 3)
         self.database_entry = gtk.Entry()
         self.database_entry.connect('changed', self.dbentry_changed)
-        self.database_entry.connect('changed', self.update_profiles, 'database')
+        self.database_entry.connect('changed', self.update_profiles,
+            'database')
         self.database_entry.set_activates_default(True)
         self.database_label = gtk.Label()
         self.database_label.set_use_markup(True)
@@ -108,7 +106,7 @@ class DBListEditor(object):
         self.database_combo.connect('changed', self.dbcombo_changed)
         self.database_button = gtk.Button(_(u'Create'))
         self.database_button.connect('clicked', self.db_create)
-        self.database_progressbar= gtk.ProgressBar()
+        self.database_progressbar = gtk.ProgressBar()
         self.database_progressbar.set_text(_(u'Fetching databases list'))
         image = gtk.Image()
         image.set_from_stock('tryton-new', gtk.ICON_SIZE_BUTTON)
@@ -131,7 +129,8 @@ class DBListEditor(object):
         username.set_alignment(1, 0.5)
         username.set_padding(3, 3)
         self.username_entry = gtk.Entry()
-        self.username_entry.connect('changed', self.update_profiles, 'username')
+        self.username_entry.connect('changed', self.update_profiles,
+            'username')
         self.username_entry.set_activates_default(True)
         table.attach(username, 0, 1, 4, 5, yoptions=False, xoptions=gtk.FILL)
         table.attach(self.username_entry, 1, 2, 4, 5, yoptions=False)
@@ -142,6 +141,16 @@ class DBListEditor(object):
         self.dialog.set_default_size(640, 350)
         self.dialog.set_default_response(gtk.RESPONSE_ACCEPT)
 
+        self.dialog.connect('close', lambda *a: False)
+        self.dialog.connect('response', self.response)
+        self.callback = callback
+
+    def response(self, widget, response):
+        if self.callback:
+            self.callback(self.current_profile['name'])
+        self.parent.present()
+        self.dialog.destroy()
+
     def run(self, profile_name):
         self.clear_entries()  # must be done before show_all for windows
         self.dialog.show_all()
@@ -154,10 +163,6 @@ class DBListEditor(object):
                 i = 0
             self.profile_tree.get_selection().select_path((i,))
             self.profile_selected(self.profile_tree)
-        self.dialog.run()
-        self.parent.present()
-        self.dialog.destroy()
-        return self.current_profile['name']
 
     def _current_profile(self):
         model, selection = self.profile_tree.get_selection().get_selected()
@@ -168,10 +173,12 @@ class DBListEditor(object):
     def clear_entries(self):
         for entryname in ('host', 'port', 'database', 'username'):
             entry = getattr(self, '%s_entry' % entryname)
+            entry.handler_block_by_func(self.update_profiles)
             if entryname == 'port':
-                entry.set_text('8070')
+                entry.set_text('8000')
             else:
                 entry.set_text('')
+            entry.handler_unblock_by_func(self.update_profiles)
         self.current_database = None
         self.database_combo.set_active(-1)
         self.database_combo.get_model().clear()
@@ -187,10 +194,10 @@ class DBListEditor(object):
     def profile_create(self, button):
         self.clear_entries()
         model = self.profile_tree.get_model()
-        selection = self.profile_tree.get_selection()
         model.append(['', False])
         column = self.profile_tree.get_column(0)
-        self.profile_tree.set_cursor(len(model)-1, column, start_editing=True)
+        self.profile_tree.set_cursor(len(model) - 1, column,
+            start_editing=True)
 
     def profile_delete(self, button):
         self.clear_entries()
@@ -225,22 +232,13 @@ class DBListEditor(object):
 
         self.display_dbwidget(None, None, self.current_database)
 
-    def edit_canceled(self, renderer):
-        model = self.profile_tree.get_model()
-        for i, row in enumerate(list(model)):
-            if not row[0]:
-                del model[i]
-
-    def check_edit_cancel(self, editable, event, renderer, path):
-        renderer.emit('edited', path, editable.get_text())
-        return False
-
     def edit_started(self, renderer, editable, path):
         if isinstance(editable, gtk.Entry):
-            editable.connect('focus-out-event', self.check_edit_cancel,
+            editable.connect('focus-out-event', self.edit_profilename,
                 renderer, path)
 
-    def edit_profilename(self, renderer, path, newtext):
+    def edit_profilename(self, editable, event, renderer, path):
+        newtext = editable.get_text()
         model = self.profile_tree.get_model()
         oldname = model[path][0]
         if oldname == newtext == '':
@@ -284,6 +282,10 @@ class DBListEditor(object):
         port = self.port_entry.get_text()
         if not (host and port):
             return
+        if (host, port, self.current_profile['name']) == self.db_cache:
+            return
+        if self.updating_db:
+            return
         if dbname is None:
             dbname = self.current_database
 
@@ -293,33 +295,38 @@ class DBListEditor(object):
         self.remove_button.set_sensitive(False)
         self.ok_button.set_sensitive(False)
         self.cell.set_property('editable', False)
+        self.host_entry.set_sensitive(False)
+        self.port_entry.set_sensitive(False)
         self.updating_db = True
-        dbs, createdb = dbprogress.update(self.database_combo,
-            self.database_progressbar, dbname)
-        self.updating_db = False
 
-        if dbs is None and createdb is None:
-            pass
-        elif dbs is None or dbs == -1:
-            if dbs is None:
-                label = _(u'Could not connect to the server')
-            else:
-                label = _(u'Incompatible version of the server')
-            self.database_label.set_label('<b>%s</b>' % label)
-            self.database_label.show()
-        elif dbs == 0:
-            if createdb:
+        def callback(dbs):
+            self.updating_db = False
+            self.db_cache = (host, port, self.current_profile['name'])
+
+            if dbs is None or dbs == -1:
+                if dbs is None:
+                    label = _(u'Could not connect to the server')
+                else:
+                    label = _(u'Incompatible version of the server')
+                self.database_label.set_label('<b>%s</b>' % label)
+                self.database_label.show()
+            elif dbs == 0:
                 self.database_button.show()
-            else:
+            elif dbs == -2:
                 self.database_entry.show()
-        else:
-            self.database_entry.set_text(dbname if dbname else '')
-            self.database_combo.show()
+            else:
+                self.database_entry.set_text(dbname if dbname else '')
+                self.database_combo.show()
 
-        self.add_button.set_sensitive(True)
-        self.remove_button.set_sensitive(True)
-        self.ok_button.set_sensitive(True)
-        self.cell.set_property('editable', True)
+            self.add_button.set_sensitive(True)
+            self.remove_button.set_sensitive(True)
+            self.ok_button.set_sensitive(True)
+            self.cell.set_property('editable', True)
+            self.host_entry.set_sensitive(True)
+            self.port_entry.set_sensitive(True)
+
+        dbprogress.update(self.database_combo,
+            self.database_progressbar, callback, dbname)
 
     def db_create(self, button):
         if not self.current_profile['name']:
@@ -327,7 +334,8 @@ class DBListEditor(object):
         host = self.host_entry.get_text()
         port = int(self.port_entry.get_text())
         dia = DBCreate(host, port)
-        dbname = dia.run(self.dialog)
+        dbname = dia.run()
+        self.db_cache = None
         self.username_entry.set_text('admin')
         self.display_dbwidget(None, None, dbname)
 
@@ -356,10 +364,11 @@ class DBListEditor(object):
 
 
 class DBLogin(object):
-    def __init__(self, parent):
+    def __init__(self):
         # GTK Stuffs
-        self.dialog = gtk.Dialog(title=_('Login'), parent=parent,
-            flags=gtk.DIALOG_MODAL|gtk.DIALOG_DESTROY_WITH_PARENT)
+        self.parent = common.get_toplevel_window()
+        self.dialog = gtk.Dialog(title=_('Login'), parent=self.parent,
+            flags=gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT)
         self.dialog.set_position(gtk.WIN_POS_CENTER_ON_PARENT)
         self.dialog.set_has_separator(True)
         self.dialog.set_icon(TRYTON_ICON)
@@ -367,7 +376,7 @@ class DBLogin(object):
         tooltips = common.Tooltips()
         button_cancel = gtk.Button(_('_Cancel'))
         img_cancel = gtk.Image()
-        img_cancel.set_from_stock('tryton-cancel', gtk.ICON_SIZE_BUTTON)
+        img_cancel.set_from_stock('gtk-cancel', gtk.ICON_SIZE_BUTTON)
         button_cancel.set_image(img_cancel)
         tooltips.set_tip(button_cancel,
             _('Cancel connection to the Tryton server'))
@@ -389,8 +398,7 @@ class DBLogin(object):
         self.dialog.vbox.pack_start(alignment, True, True, 0)
 
         image = gtk.Image()
-        image.set_from_file(os.path.join(PIXMAPS_DIR,
-            'tryton.png').decode('utf-8'))
+        image.set_from_file(os.path.join(PIXMAPS_DIR, 'tryton.png'))
         image.set_alignment(0.5, 1)
         ebox = gtk.EventBox()
         ebox.modify_bg(gtk.STATE_NORMAL, gtk.gdk.color_parse("#1b2019"))
@@ -406,8 +414,6 @@ class DBLogin(object):
         self.combo_profile.add_attribute(cell, 'sensitive', 1)
         self.combo_profile.set_model(self.profile_store)
         self.combo_profile.connect('changed', self.profile_changed)
-        self.combo_profile.connect('move-active', self.profile_move_active)
-        self.move_active = False
         self.profile_label = gtk.Label(_(u'Profile:'))
         self.profile_label.set_justify(gtk.JUSTIFY_RIGHT)
         self.profile_label.set_alignment(1, 0.5)
@@ -430,7 +436,9 @@ class DBLogin(object):
         self.label_host.set_alignment(1, 0.5)
         self.label_host.set_padding(3, 3)
         self.entry_host = gtk.Entry()
-        self.entry_host.connect_after('focus-out-event', self.clear_profile_combo)
+        self.entry_host.connect_after('focus-out-event',
+            self.clear_profile_combo)
+        self.entry_host.set_activates_default(True)
         self.table_main.attach(self.label_host, 0, 1, 4, 5, xoptions=gtk.FILL)
         self.table_main.attach(self.entry_host, 1, 3, 4, 5)
         self.label_database = gtk.Label(_('Database:'))
@@ -438,7 +446,9 @@ class DBLogin(object):
         self.label_database.set_alignment(1, 0.5)
         self.label_database.set_padding(3, 3)
         self.entry_database = gtk.Entry()
-        self.entry_database.connect_after('focus-out-event', self.clear_profile_combo)
+        self.entry_database.connect_after('focus-out-event',
+            self.clear_profile_combo)
+        self.entry_database.set_activates_default(True)
         self.table_main.attach(self.label_database, 0, 1, 5, 6,
             xoptions=gtk.FILL)
         self.table_main.attach(self.entry_database, 1, 3, 5, 6)
@@ -449,25 +459,25 @@ class DBLogin(object):
         self.entry_login = gtk.Entry()
         self.entry_login.set_activates_default(True)
         self.table_main.attach(self.entry_login, 1, 3, 6, 7)
-        label_password = gtk.Label(str = _("Password:"))
+        label_password = gtk.Label(str=_("Password:"))
         label_password.set_justify(gtk.JUSTIFY_RIGHT)
         label_password.set_alignment(1, 0.5)
         label_password.set_padding(3, 3)
         self.table_main.attach(label_password, 0, 1, 7, 8, xoptions=gtk.FILL)
-        label_username = gtk.Label(str = _("User name:"))
+        label_username = gtk.Label(str=_("User name:"))
         label_username.set_alignment(1, 0.5)
         label_username.set_padding(3, 3)
         self.table_main.attach(label_username, 0, 1, 6, 7, xoptions=gtk.FILL)
 
         # Profile informations
         self.profile_cfg = os.path.join(get_config_dir(), 'profiles.cfg')
-        self.profiles = ConfigParser.SafeConfigParser({'port': '8070'})
+        self.profiles = ConfigParser.SafeConfigParser({'port': '8000'})
         if not os.path.exists(self.profile_cfg):
             short_version = '.'.join(VERSION.split('.', 2)[:2])
             name = 'demo%s.tryton.org' % short_version
             self.profiles.add_section(name)
             self.profiles.set(name, 'host', name)
-            self.profiles.set(name, 'port', '8070')
+            self.profiles.set(name, 'port', '8000')
             self.profiles.set(name, 'database', 'demo%s' % short_version)
             self.profiles.set(name, 'username', 'demo')
         else:
@@ -478,20 +488,23 @@ class DBLogin(object):
             self.profile_store.append([section, active])
 
     def profile_manage(self, widget):
-        dia = DBListEditor(self.dialog, self.profile_store, self.profiles)
+        def callback(profile_name):
+            with open(self.profile_cfg, 'wb') as configfile:
+                self.profiles.write(configfile)
+
+            for idx, row in enumerate(self.profile_store):
+                if row[0] == profile_name and row[1]:
+                    self.combo_profile.set_active(idx)
+                    self.profile_changed(self.combo_profile)
+                    break
+
+        dia = DBListEditor(self.dialog, self.profile_store, self.profiles,
+            callback)
         active_profile = self.combo_profile.get_active()
         profile_name = None
         if active_profile != -1:
             profile_name = self.profile_store[active_profile][0]
-        profile_name = dia.run(profile_name)
-        with open(self.profile_cfg, 'wb') as configfile:
-            self.profiles.write(configfile)
-
-        for idx, row in enumerate(self.profile_store):
-            if row[0] == profile_name and row[1]:
-                self.combo_profile.set_active(idx)
-                self.profile_changed(self.combo_profile)
-                break
+        dia.run(profile_name)
 
     def profile_changed(self, combobox):
         position = combobox.get_active()
@@ -510,22 +523,14 @@ class DBLogin(object):
         self.entry_database.set_text(self.profiles.get(profile, 'database'))
         if username:
             self.entry_login.set_text(username)
-            focus_widget = self.entry_password
         else:
             self.entry_login.set_text('')
-            focus_widget = self.entry_login
-        if not self.move_active:
-            focus_widget.grab_focus()
-        self.move_active = False
-
-    def profile_move_active(self, combobox, scrolltype):
-        self.move_active = True
 
     def clear_profile_combo(self, entry, event):
         netloc = self.entry_host.get_text()
-        host = self.get_hostname(netloc)
+        host = common.get_hostname(netloc)
         try:
-            port = str(self.get_port(netloc))
+            port = str(common.get_port(netloc))
         except ValueError:
             host = ''
             port = ''
@@ -541,6 +546,7 @@ class DBLogin(object):
         else:
             idx = -1
         self.combo_profile.set_active(idx)
+        return False
 
     def expand_hostspec(self, expander, *args):
         visibility = expander.props.expanded
@@ -549,29 +555,14 @@ class DBLogin(object):
         self.entry_database.props.visible = visibility
         self.label_database.props.visible = visibility
 
-    def get_hostname(self, netloc):
-        if '[' in netloc and ']' in netloc:
-            return netloc.split(']')[0][1:]
-        elif ':' in netloc:
-            return netloc.split(':')[0]
-        else:
-            return netloc
-
-    def get_port(self, netloc):
-        netloc = netloc.split(']')[-1]
-        if ':' in netloc:
-            return int(netloc.split(':')[1])
-        else:
-            return 8070
-
-    def run(self, parent):
+    def run(self):
         profile_name = CONFIG['login.profile']
         can_use_profile = self.profiles.has_section(profile_name)
         if can_use_profile:
             for (configname, sectionname) in (('login.server', 'host'),
                     ('login.port', 'port'), ('login.db', 'database')):
-                if (self.profiles.get(profile_name, sectionname) != \
-                        CONFIG[configname]):
+                if (self.profiles.get(profile_name, sectionname)
+                        != CONFIG[configname]):
                     can_use_profile = False
                     break
 
@@ -614,16 +605,18 @@ class DBLogin(object):
                 profile = self.profile_store[active_profile][0]
                 CONFIG['login.profile'] = profile
             netloc = self.entry_host.get_text()
-            host = self.get_hostname(netloc)
+            host = common.get_hostname(netloc)
             try:
-                port = self.get_port(netloc)
+                port = common.get_port(netloc)
             except ValueError:
                 continue
-            if ':' in host:
-                hostname = '[%s]' % host
-            else:
-                hostname = host
-            if not common.test_server_version(host, port):
+            try:
+                if not common.test_server_version(host, port):
+                    common.warning('',
+                        _(u'Incompatible version of the server'))
+                    continue
+            except Exception, exception:
+                common.process_exception(exception)
                 continue
             database = self.entry_database.get_text()
             login = self.entry_login.get_text()
@@ -635,14 +628,9 @@ class DBLogin(object):
             result = (self.entry_login.get_text(),
                 self.entry_password.get_text(), host, port, database)
 
-        if res != gtk.RESPONSE_OK:
-            parent.present()
-            self.dialog.destroy()
-            rpc.logout()
-            from tryton.gui.main import Main
-            Main.get_main().refresh_ssl()
-            raise Exception('QueryCanceled')
-        parent.present()
+        self.parent.present()
         self.dialog.destroy()
+        if res != gtk.RESPONSE_OK:
+            rpc.logout()
+            raise TrytonError('QueryCanceled')
         return result
-
