@@ -4,25 +4,21 @@
 import gettext
 import gtk
 import gobject
-import locale
-import tryton.rpc as rpc
 from tryton.gui.window.view_form.screen import Screen
 from tryton.action import Action
-from tryton.config import CONFIG
 from tryton.gui import Main
 from tryton.gui.window import Window
-from tryton.gui.window.preference import Preference
 from tryton.gui.window.win_export import WinExport
 from tryton.gui.window.win_import import WinImport
 from tryton.gui.window.attachment import Attachment
+from tryton.gui.window.revision import Revision
 from tryton.signal_event import SignalEvent
-from tryton.common import (TRYTON_ICON, message, sur, sur_3b, COLOR_SCHEMES,
-    timezoned_date)
-from tryton.exceptions import TrytonServerError
+from tryton.common import message, sur, sur_3b, COLOR_SCHEMES, timezoned_date
 import tryton.common as common
 from tryton.translate import date_format
-from tryton.common import DT_FORMAT, DHM_FORMAT, HM_FORMAT
-from tryton.pyson import PYSONEncoder
+from tryton.common import RPCExecute, RPCException
+from tryton.common.datetime_strftime import datetime_strftime
+from tryton import plugins
 
 from tabcontent import TabContent
 
@@ -53,30 +49,32 @@ class Form(SignalEvent, TabContent):
     menu_def = [
         (_('_New'), 'tryton-new', 'sig_new', '<tryton>/Form/New'),
         (_('_Save'), 'tryton-save', 'sig_save', '<tryton>/Form/Save'),
+        (_('_Switch View'), 'tryton-fullscreen', 'sig_switch',
+            '<tryton>/Form/Switch View'),
+        (_('_Reload/Undo'), 'tryton-refresh', 'sig_reload',
+            '<tryton>/Form/Reload'),
         (_('_Duplicate'), 'tryton-copy', 'sig_copy',
             '<tryton>/Form/Duplicate'),
         (_('_Delete...'), 'tryton-delete', 'sig_remove',
             '<tryton>/Form/Delete'),
         (None,) * 4,
-        (_('_Next'), 'tryton-go-next', 'sig_next', '<tryton>/Form/Next'),
         (_('_Previous'), 'tryton-go-previous', 'sig_previous',
             '<tryton>/Form/Previous'),
-        (_('_Switch View'), 'tryton-fullscreen', 'sig_switch',
-            '<tryton>/Form/Switch View'),
+        (_('_Next'), 'tryton-go-next', 'sig_next', '<tryton>/Form/Next'),
+        (_('_Search'), 'tryton-find', 'sig_search', '<tryton>/Form/Search'),
+        (_('View _Logs...'), None, 'sig_logs', None),
+        (_('Show revisions...'), 'tryton-clock', 'revision', None),
         (None,) * 4,
         (_('_Close Tab'), 'tryton-close', 'sig_win_close',
             '<tryton>/Form/Close'),
         (None,) * 4,
-        (_('View _Logs...'), None, 'sig_logs', None),
-        (_('_Go to Record ID...'), None, 'sig_goto', '<tryton>/Form/Goto'),
-        (None,) * 4,
-        (_('_Reload/Undo'), 'tryton-refresh', 'sig_reload',
-            '<tryton>/Form/Reload'),
-        (None,) * 4,
+        (_('A_ttachments...'), 'tryton-attachment', 'sig_attach',
+            '<tryton>/Form/Attachments'),
         (_('_Actions...'), 'tryton-executable', 'sig_action',
             '<tryton>/Form/Actions'),
         (_('_Relate...'), 'tryton-go-jump', 'sig_relate',
             '<tryton>/Form/Relate'),
+        (None,) * 4,
         (_('_Report...'), 'tryton-print-open', 'sig_print_open',
             '<tryton>/Form/Report'),
         (_('_E-Mail...'), 'tryton-print-email', 'sig_print_email',
@@ -90,9 +88,9 @@ class Form(SignalEvent, TabContent):
             '<tryton>/Form/Import Data'),
     ]
 
-    def __init__(self, model, res_id=False, domain=None, mode=None,
+    def __init__(self, model, res_id=False, domain=None, order=None, mode=None,
             view_ids=None, context=None, name=False, limit=None,
-            auto_refresh=False, search_value=None):
+            search_value=None, tab_domain=None):
         super(Form, self).__init__()
 
         if not mode:
@@ -101,21 +99,18 @@ class Form(SignalEvent, TabContent):
             domain = []
         if view_ids is None:
             view_ids = []
-        if context is None:
-            context = {}
 
         self.model = model
         self.res_id = res_id
         self.domain = domain
         self.mode = mode
         self.context = context
-        self.auto_refresh = auto_refresh
         self.view_ids = view_ids
         self.dialogs = []
 
-        self.screen = Screen(self.model, mode=mode, context=self.context,
-            view_ids=view_ids, domain=domain, limit=limit,
-            readonly=bool(auto_refresh), search_value=search_value)
+        self.screen = Screen(self.model, mode=mode, context=context,
+            view_ids=view_ids, domain=domain, limit=limit, order=order,
+            search_value=search_value, tab_domain=tab_domain)
         self.screen.widget.show()
 
         if not name:
@@ -123,11 +118,34 @@ class Form(SignalEvent, TabContent):
         else:
             self.name = name
 
+        if self.model not in common.MODELHISTORY:
+            self.menu_def = self.menu_def[:]
+            # Remove callback to revision
+            self.menu_def[11] = (self.menu_def[11][:2] + (None,)
+                + self.menu_def[11][3:])
+
         self.create_tabcontent()
+
+        self.url_entry = url_entry = gtk.Entry()
+        url_entry.show()
+        url_entry.set_editable(False)
+        style = url_entry.get_style()
+        url_entry.modify_bg(gtk.STATE_ACTIVE,
+            style.bg[gtk.STATE_INSENSITIVE])
+        self.widget.pack_start(url_entry, False, False)
+
+        access = common.MODELACCESS[self.model]
+        for button, access_type in (
+                ('new', 'create'),
+                ('save', 'write'),
+                ):
+            self.buttons[button].props.sensitive = access[access_type]
+
         self.screen.signal_connect(self, 'record-message',
             self._record_message)
+
         self.screen.signal_connect(self, 'record-modified',
-            self._record_modified)
+            lambda *a: gobject.idle_add(self._record_modified, *a))
         self.screen.signal_connect(self, 'record-saved', self._record_saved)
         self.screen.signal_connect(self, 'attachment-count',
                 self._attachment_count)
@@ -143,20 +161,14 @@ class Form(SignalEvent, TabContent):
                     in ('tree', 'graph', 'calendar'):
                 self.screen.search_filter()
 
-        if auto_refresh and int(auto_refresh):
-            gobject.timeout_add(int(auto_refresh) * 1000, self.sig_reload)
+        self.update_revision()
 
     def get_toolbars(self):
-        ctx = {}
-        ctx.update(rpc.CONTEXT)
-        ctx.update(self.context)
-        args = ('model', self.model, 'view_toolbar_get', ctx)
         try:
-            toolbars = rpc.execute(*args)
-        except TrytonServerError, exception:
-            toolbars = common.process_exception(exception, *args)
-            toolbars = toolbars if toolbars else {}
-        return toolbars
+            return RPCExecute('model', self.model, 'view_toolbar_get',
+                context=self.screen.context)
+        except RPCException:
+            return {}
 
     def widget_get(self):
         return self.screen.widget
@@ -171,64 +183,13 @@ class Form(SignalEvent, TabContent):
             and self.domain == value.domain
             and self.mode == value.mode
             and self.view_ids == value.view_ids
-            and self.context == value.context
+            and self.screen.context == value.screen.context
             and self.name == value.name
             and self.screen.limit == value.screen.limit
-            and self.auto_refresh == value.auto_refresh
             and self.screen.search_value == value.screen.search_value)
 
-    def sig_goto(self, widget=None):
-        if not self.modified_save():
-            return
-        parent = common.get_toplevel_window()
-        win = gtk.Dialog(_('Go to ID'), parent,
-                gtk.DIALOG_MODAL|gtk.DIALOG_DESTROY_WITH_PARENT,
-                (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
-                    gtk.STOCK_OK, gtk.RESPONSE_OK))
-        win.set_icon(TRYTON_ICON)
-        win.set_has_separator(True)
-        win.set_default_response(gtk.RESPONSE_OK)
-
-        table = gtk.Table(2, 2)
-        table.attach(gtk.Label(_('Go to ID:')), 1, 2, 0, 1, gtk.FILL)
-        img = gtk.Image()
-        img.set_from_stock('tryton-go-jump', gtk.ICON_SIZE_DIALOG)
-        table.attach(img, 0, 1, 0, 2, gtk.FILL)
-
-        entry = gtk.Entry()
-        entry.set_property('activates_default', True)
-        entry.set_max_length(0)
-        entry.set_alignment(1.0)
-        def sig_insert_text(widget, new_text, new_text_length, position):
-            value = widget.get_text()
-            position = widget.get_position()
-            new_value = value[:position] + new_text + value[position:]
-            try:
-                locale.atoi(new_value)
-            except ValueError:
-                widget.stop_emission('insert-text')
-        entry.connect('insert_text', sig_insert_text)
-        table.attach(entry, 1, 2, 1, 2)
-
-        win.vbox.pack_start(table, expand=True, fill=True)
-        win.show_all()
-
-        response = win.run()
-        if response == gtk.RESPONSE_OK:
-            self.screen.display(locale.atoi(entry.get_text()), set_cursor=True)
-        win.destroy()
-        parent.present()
-
     def destroy(self):
-        self.screen.signal_unconnect(self)
         self.screen.destroy()
-        self.screen = None
-        self.widget = None
-        #self.scrolledwindow.destroy()
-        #self.scrolledwindow = None
-
-    def sel_ids_get(self):
-        return self.screen.sel_ids_get()
 
     def ids_get(self):
         return self.screen.ids_get()
@@ -237,13 +198,11 @@ class Form(SignalEvent, TabContent):
         return self.screen.id_get()
 
     def sig_attach(self, widget=None):
-        obj_id = self.id_get()
-        if obj_id >= 0 and obj_id is not False:
-            Attachment(self.model, obj_id).run()
-        else:
-            self.message_info(_('No record selected!'))
-        self.update_attachment_count(reload=True)
-        return True
+        record = self.screen.current_record
+        if not record or record.id < 0:
+            return
+        Attachment(record,
+            lambda: self.update_attachment_count(reload=True))
 
     def update_attachment_count(self, reload=False):
         record = self.screen.current_record
@@ -260,6 +219,9 @@ class Form(SignalEvent, TabContent):
             self.buttons['attach'].set_stock_id('tryton-attachment-hi')
         else:
             self.buttons['attach'].set_stock_id('tryton-attachment')
+        record_id = self.id_get()
+        self.buttons['attach'].props.sensitive = bool(
+            record_id >= 0 and record_id is not False)
 
     def sig_switch(self, widget=None):
         if not self.modified_save():
@@ -280,31 +242,68 @@ class Form(SignalEvent, TabContent):
             ('write_date', _('Latest Modification Date:')),
         ]
 
-        ctx = self.context.copy()
-        ctx.update(rpc.CONTEXT)
-        args = ('model', self.model, 'read', [obj_id], [x[0] for x in fields],
-                ctx)
         try:
-            res = rpc.execute(*args)
-        except TrytonServerError, exception:
-            res = common.process_exception(exception, *args)
-            if not res:
-                return
+            res = RPCExecute('model', self.model, 'read', [obj_id],
+                [x[0] for x in fields], context=self.screen.context)
+        except RPCException:
+            return
         message_str = ''
         for line in res:
             for (key, val) in fields:
                 value = str(line.get(key, False) or '/')
                 if line.get(key, False) \
                         and key in ('create_date', 'write_date'):
-                    display_format = date_format() + ' ' + HM_FORMAT
+                    display_format = date_format() + ' %H:%M:%S'
                     date = timezoned_date(line[key])
                     value = common.datetime_strftime(date, display_format)
-                message_str += val + ' ' + value +'\n'
+                message_str += val + ' ' + value + '\n'
         message_str += _('Model:') + ' ' + self.model
         message(message_str)
         return True
 
+    def revision(self, widget=None):
+        if not self.modified_save():
+            return
+        current_id = (self.screen.current_record.id
+            if self.screen.current_record else None)
+        try:
+            revisions = RPCExecute('model', self.model, 'history_revisions',
+                [r.id for r in self.screen.selected_records])
+        except RPCException:
+            return
+        revision = self.screen.context.get('_datetime')
+        revision = Revision(revisions, revision).run()
+        # Prevent too old revision in form view
+        if (self.screen.current_view.view_type == 'form'
+                and revision < revisions[-1][0]):
+                revision = revisions[-1][0]
+        if revision != self.screen.context.get('_datetime'):
+            self.screen.clear()
+            # Update root group context that will be propagated
+            self.screen.group._context['_datetime'] = revision
+            if self.screen.current_view.view_type != 'form':
+                self.screen.search_filter(
+                    self.screen.screen_container.get_text())
+            else:
+                # Test if record exist in revisions
+                self.screen.load([current_id])
+            self.screen.display(set_cursor=True)
+            self.update_revision()
+
+    def update_revision(self):
+        revision = self.screen.context.get('_datetime')
+        if revision:
+            format_ = date_format() + ' %H:%M:%S.%f'
+            revision = datetime_strftime(revision, format_)
+            self.title.set_label('%s @ %s' % (self.name, revision))
+        else:
+            self.title.set_label(self.name)
+        for button in ('new', 'save'):
+            self.buttons[button].props.sensitive = not revision
+
     def sig_remove(self, widget=None):
+        if not common.MODELACCESS[self.model]['delete']:
+            return
         if self.screen.current_view.view_type == 'form':
             msg = _('Are you sure to remove this record?')
         else:
@@ -316,25 +315,17 @@ class Form(SignalEvent, TabContent):
                 self.message_info(_('Records removed!'), 'green')
 
     def sig_import(self, widget=None):
-        while(self.screen.view_to_load):
-            self.screen.load_view_to_load()
-        fields = {}
-        for name, field in self.screen.group.fields.iteritems():
-            fields[name] = field.attrs
-        win = WinImport(self.model)
-        win.run()
+        WinImport(self.model, self.screen.context)
 
     def sig_save_as(self, widget=None):
-        while self.screen.view_to_load:
-            self.screen.load_view_to_load()
-        fields = {}
-        for name, field in self.screen.group.fields.iteritems():
-            fields[name] = field.attrs
-        win = WinExport(self.model, self.ids_get(),
-                context=self.context)
-        win.run()
+        export = WinExport(self.model, self.ids_get(),
+            context=self.screen.context)
+        for name in self.screen.current_view.get_fields():
+            export.sel_field(name)
 
     def sig_new(self, widget=None, autosave=True):
+        if not common.MODELACCESS[self.model]['create']:
+            return
         if autosave:
             if not self.modified_save():
                 return
@@ -343,22 +334,20 @@ class Form(SignalEvent, TabContent):
         self.activate_save()
 
     def sig_copy(self, widget=None):
+        if not common.MODELACCESS[self.model]['create']:
+            return
         if not self.modified_save():
             return
-        res_ids = self.sel_ids_get()
-        ctx = self.context.copy()
-        ctx.update(rpc.CONTEXT)
-        args = ('model', self.model, 'copy', res_ids, {}, ctx)
-        try:
-            new_ids = rpc.execute(*args)
-        except TrytonServerError, exception:
-            new_ids = common.process_exception(exception, *args)
-        if new_ids:
-            self.screen.load(new_ids)
-            self.message_info(_('Working now on the duplicated record(s)!'),
-                    'green')
+        self.screen.copy()
+        self.message_info(_('Working now on the duplicated record(s)!'),
+            'green')
 
     def sig_save(self, widget=None):
+        if widget:
+            # Called from button so we must save the tree state
+            self.screen.save_tree_state()
+        if not common.MODELACCESS[self.model]['write']:
+            return
         if self.screen.save_current():
             self.message_info(_('Record saved!'), 'green')
             return True
@@ -371,54 +360,61 @@ class Form(SignalEvent, TabContent):
             return
         self.screen.display_prev()
         self.message_info('')
+        self.activate_save()
 
     def sig_next(self, widget=None):
         if not self.modified_save():
             return
         self.screen.display_next()
         self.message_info('')
+        self.activate_save()
 
     def sig_reload(self, test_modified=True):
-        if not hasattr(self, 'screen'):
-            return False
-        if test_modified and self.screen.modified():
-            res = sur_3b(_('This record has been modified\n' \
-                    'do you want to save it ?'))
-            if res == 'ok':
-                self.sig_save(None)
-            elif res == 'ko':
-                pass
-            else:
+        if test_modified:
+            if not self.modified_save():
                 return False
-        if self.screen.current_view.view_type == 'form':
-            self.screen.cancel_current()
-            self.screen.display()
         else:
-            obj_id = self.id_get()
+            self.screen.save_tree_state(store=False)
+        self.screen.cancel_current()
+        set_cursor = False
+        record_id = (self.screen.current_record.id
+            if self.screen.current_record else None)
+        if self.screen.current_view.view_type != 'form':
             self.screen.search_filter(self.screen.screen_container.get_text())
             for record in self.screen.group:
-                if record.id == obj_id:
+                if record.id == record_id:
                     self.screen.current_record = record
-                    self.screen.display(set_cursor=True)
+                    set_cursor = True
                     break
+        self.screen.display(set_cursor=set_cursor)
         self.message_info('')
         self.activate_save()
         return True
 
     def sig_action(self, widget):
-        self.buttons['action'].props.active = True
+        if self.buttons['action'].props.sensitive:
+            self.buttons['action'].props.active = True
 
     def sig_print(self, widget):
-        self.buttons['print'].props.active = True
+        if self.buttons['print'].props.sensitive:
+            self.buttons['print'].props.active = True
 
     def sig_print_open(self, widget):
-        self.buttons['open'].props.active = True
+        if self.buttons['open'].props.sensitive:
+            self.buttons['open'].props.active = True
 
     def sig_print_email(self, widget):
-        self.buttons['email'].props.active = True
+        if self.buttons['email'].props.sensitive:
+            self.buttons['email'].props.active = True
 
     def sig_relate(self, widget):
-        self.buttons['relate'].props.active = True
+        if self.buttons['relate'].props.sensitive:
+            self.buttons['relate'].props.active = True
+
+    def sig_search(self, widget):
+        search_container = self.screen.screen_container
+        if hasattr(search_container, 'search_entry'):
+            search_container.search_entry.grab_focus()
 
     def action_popup(self, widget):
         button, = widget.get_children()
@@ -454,12 +450,14 @@ class Form(SignalEvent, TabContent):
         name = '_'
         if signal_data[0]:
             name = str(signal_data[0])
-        for button_id in ('print', 'action', 'relate', 'email', 'open', 'save',
-                'switch', 'attach'):
+        for button_id in ('print', 'relate', 'email', 'open', 'save',
+                'attach'):
             button = self.buttons[button_id]
             can_be_sensitive = getattr(button, '_can_be_sensitive', True)
             button.props.sensitive = (bool(signal_data[0])
                 and can_be_sensitive)
+        button_switch = self.buttons['switch']
+        button_switch.props.sensitive = self.screen.number_of_views > 1
 
         msg = name + ' / ' + str(signal_data[1])
         if signal_data[1] < signal_data[2]:
@@ -467,71 +465,51 @@ class Form(SignalEvent, TabContent):
         self.status_label.set_text(msg)
         self.message_info('')
         self.activate_save()
+        self.url_entry.set_text(self.screen.get_url())
 
     def _record_modified(self, screen, signal_data):
-        self.message_info('', color='white')
-        self.activate_save()
+        # As it is called via idle_add, the form could have been destroyed in
+        # the meantime.
+        if self.widget_get().props.window:
+            self.activate_save()
 
     def _record_saved(self, screen, signal_data):
         self.activate_save()
+        self.update_attachment_count()
 
-    def modified_save(self, reload=True):
+    def modified_save(self):
+        self.screen.save_tree_state()
         self.screen.current_view.set_value()
         if self.screen.modified():
             value = sur_3b(
-                    _('This record has been modified\n' \
-                            'do you want to save it ?'))
+                _('This record has been modified\n'
+                    'do you want to save it ?'))
             if value == 'ok':
                 return self.sig_save(None)
-            elif value == 'ko':
-                if reload:
-                    self.sig_reload(test_modified=False)
-                return True
-            else:
-                return False
+            if value == 'ko':
+                return self.sig_reload(test_modified=False)
+            return False
         return True
 
     def sig_close(self, widget=None):
         for dialog in self.dialogs[:]:
             dialog.destroy()
-        return self.modified_save(reload=False)
+        return self.modified_save()
 
     def _action(self, action, atype):
         action = action.copy()
         if not self.screen.save_current():
             return
-        record_id = self.screen.id_get()
-        record_ids = self.screen.sel_ids_get()
-        if atype in ('print', 'action'):
-            email = {}
-            if 'pyson_email' in action:
-                email = self.screen.current_record.expr_eval(
-                    action['pyson_email'])
-                if not email:
-                    email = {}
-            if 'subject' not in email:
-                email['subject'] = action['name'].replace('_', '')
-            action['email'] = email
-            self.screen.display()
-        elif atype == 'relate':
-            encoder = PYSONEncoder()
-            if 'pyson_domain' in action:
-                action['pyson_domain'] = encoder.encode(
-                        self.screen.current_record.expr_eval(
-                                action['pyson_domain'], check_load=False))
-            if 'pyson_context' in action:
-                action['pyson_context'] = encoder.encode(
-                        self.screen.current_record.expr_eval(
-                            action['pyson_context'], check_load=False))
-        else:
-            raise NotImplementedError("Action type '%s' is not supported" %
-                atype)
+        record_id = (self.screen.current_record.id
+            if self.screen.current_record else None)
+        record_ids = [r.id for r in self.screen.selected_records]
+        action = Action.evaluate(action, atype, self.screen.current_record)
         data = {
             'model': self.screen.model_name,
             'id': record_id,
             'ids': record_ids,
         }
-        Action._exec_action(action, data, {})
+        Action._exec_action(action, data, self.screen.context)
 
     def activate_save(self):
         self.buttons['save'].props.sensitive = self.screen.modified()
@@ -542,6 +520,13 @@ class Form(SignalEvent, TabContent):
     def create_toolbar(self, toolbars):
         gtktoolbar = super(Form, self).create_toolbar(toolbars)
 
+        attach_btn = self.buttons['attach']
+        attach_btn.drag_dest_set(gtk.DEST_DEFAULT_ALL, [
+                ('text/uri-list', 0, 0),
+                ], gtk.gdk.ACTION_MOVE)
+        attach_btn.connect('drag_data_received',
+            self.attach_drag_data_received)
+
         iconstock = {
             'print': 'tryton-print',
             'action': 'tryton-executable',
@@ -565,7 +550,9 @@ class Form(SignalEvent, TabContent):
                 tbutton.connect('toggled', self.action_popup)
                 self.tooltips.set_tip(tbutton, tooltip)
                 self.buttons[special_action] = tbutton
-                tbutton._can_be_sensitive = bool(toolbars.get(action_type))
+                if action_type != 'action':
+                    tbutton._can_be_sensitive = bool(
+                        tbutton._menu.get_children())
             else:
                 tbutton = gtk.SeparatorToolItem()
             gtktoolbar.insert(tbutton, -1)
@@ -576,6 +563,9 @@ class Form(SignalEvent, TabContent):
         menu = gtk.Menu()
         menu.connect('deactivate', self._popup_menu_hide, widget)
 
+        if keyword == 'action':
+            widget.connect('toggled', self._update_action_popup, menu)
+
         for action in actions:
             new_action = action.copy()
             if special_action == 'print':
@@ -590,7 +580,6 @@ class Form(SignalEvent, TabContent):
             menuitem.connect('activate', self._popup_menu_selected, widget,
                 new_action, keyword)
             menu.add(menuitem)
-
         return menu
 
     def _popup_menu_selected(self, menuitem, togglebutton, action, keyword):
@@ -605,3 +594,53 @@ class Form(SignalEvent, TabContent):
 
     def _popup_menu_hide(self, menuitem, togglebutton):
         togglebutton.props.active = False
+
+    def _update_action_popup(self, tbutton, menu):
+        for item in menu.get_children():
+            if (getattr(item, '_update_action', False)
+                    or isinstance(item, gtk.SeparatorMenuItem)):
+                menu.remove(item)
+
+        buttons = self.screen.get_buttons()
+        if buttons:
+            menu.add(gtk.SeparatorMenuItem())
+        for button in buttons:
+            menuitem = gtk.ImageMenuItem(button.attrs.get('icon'))
+            menuitem.set_label('_' + button.attrs.get('string', _('Unknown')))
+            menuitem.set_use_underline(True)
+            menuitem.connect('activate',
+                lambda m, attrs: self.screen.button(attrs), button.attrs)
+            menuitem._update_action = True
+            menu.add(menuitem)
+
+        menu.add(gtk.SeparatorMenuItem())
+        for plugin in plugins.MODULES:
+            for name, func in plugin.get_plugins(self.model):
+                menuitem = gtk.MenuItem('_' + name)
+                menuitem.set_use_underline(True)
+                menuitem.connect('activate', lambda m, func: func({
+                            'model': self.model,
+                            'ids': [r.id
+                                for r in self.screen.selected_records],
+                            'id': self.id_get(),
+                            }), func)
+                menuitem._update_action = True
+                menu.add(menuitem)
+
+    def set_cursor(self):
+        if self.screen:
+            self.screen.set_cursor(reset_view=False)
+
+    def attach_drag_data_received(self, widget, context, x, y, selection, info,
+            timestamp):
+        record = self.screen.current_record
+        if not record or record.id < 0:
+            return
+        win_attach = Attachment(record,
+            lambda: self.update_attachment_count(reload=True))
+        if info == 0:
+            for uri in selection.data.splitlines():
+                # Win32 cut&paste terminates the list with a NULL character
+                if not uri or uri == '\0':
+                    continue
+                win_attach.add_uri(uri)
