@@ -37,13 +37,14 @@ class Field(object):
     def sig_changed(self, record):
         if self.get_state_attrs(record).get('readonly', False):
             return
-        if self.attrs.get('on_change', False):
-            record.on_change(self.name, self.attrs['on_change'])
-        record.on_change_with(self.name)
+        record.on_change([self.name])
+        record.on_change_with([self.name])
         record.autocomplete_with(self.name)
+        record.set_field_context()
 
-    def domains_get(self, record):
-        screen_domain = domain_inversion(record.group.domain4inversion,
+    def domains_get(self, record, pre_validate=None):
+        screen_domain = domain_inversion(
+            [record.group.domain4inversion, pre_validate or []],
             self.name, EvalEnvironment(record))
         if isinstance(screen_domain, bool) and not screen_domain:
             screen_domain = [('id', '=', None)]
@@ -56,8 +57,8 @@ class Field(object):
         screen_domain, attr_domain = self.domains_get(record)
         return concat(localize_domain(screen_domain), attr_domain)
 
-    def validation_domains(self, record):
-        return concat(*self.domains_get(record))
+    def validation_domains(self, record, pre_validate=None):
+        return concat(*self.domains_get(record, pre_validate))
 
     def context_get(self, record):
         context = record.context_get().copy()
@@ -74,12 +75,12 @@ class Field(object):
                 return False
         return True
 
-    def validate(self, record, softvalidation=False):
+    def validate(self, record, softvalidation=False, pre_validate=None):
         if self.attrs.get('readonly'):
             return True
         res = True
         self.get_state_attrs(record)['domain_readonly'] = False
-        domain = simplify(self.validation_domains(record))
+        domain = simplify(self.validation_domains(record, pre_validate))
         if not softvalidation:
             res = res and self.check_required(record)
         if isinstance(domain, bool):
@@ -155,7 +156,7 @@ class Field(object):
 
     def set_on_change(self, record, value):
         record.modified_fields.setdefault(self.name)
-        return self.set(record, value)
+        self.set(record, value)
 
     def state_set(self, record, states=('readonly', 'required', 'invisible')):
         state_changes = record.expr_eval(self.attrs.get('states', {}))
@@ -377,13 +378,6 @@ class M2OField(Field):
 
     _default = None
 
-    def get(self, record):
-        value = record.value.get(self.name)
-        if (record.parent_name == self.name
-                and self.attrs['relation'] == record.group.parent.model_name):
-            value = record.parent.id if record.parent else None
-        return value
-
     def get_client(self, record):
         rec_name = record.value.get(self.name + '.rec_name')
         if rec_name is None:
@@ -405,30 +399,15 @@ class M2OField(Field):
 
     def set(self, record, value):
         rec_name = record.value.get(self.name + '.rec_name') or ''
-        if value is False:
-            value = None
-        if (record.parent_name == self.name
-                and self.attrs['relation'] == record.group.parent.model_name):
-            if record.parent:
-                value = record.parent.id
-                if 'rec_name' in record.parent.value:
-                    rec_name = record.parent.value['rec_name'] or ''
-            else:
-                value = None
         if not rec_name and value >= 0:
             try:
                 result, = RPCExecute('model', self.attrs['relation'], 'read',
                     [value], ['rec_name'])
             except RPCException:
-                return False
+                return
             rec_name = result['rec_name'] or ''
         record.value[self.name + '.rec_name'] = rec_name
         record.value[self.name] = value
-        if (record.parent_name == self.name
-                and self.attrs['relation'] == record.group.parent.model_name):
-            if record.parent:
-                if 'rec_name' not in record.parent.value:
-                    record.parent.value['rec_name'] = rec_name
 
     def context_get(self, record):
         context = super(M2OField, self).context_get(record)
@@ -437,8 +416,8 @@ class M2OField(Field):
                 )[self.attrs.get('datetime_field')]
         return context
 
-    def validation_domains(self, record):
-        screen_domain, attr_domain = self.domains_get(record)
+    def validation_domains(self, record, pre_validate=None):
+        screen_domain, attr_domain = self.domains_get(record, pre_validate)
         return screen_domain
 
     def domain_get(self, record):
@@ -446,13 +425,11 @@ class M2OField(Field):
         return concat(localize_domain(inverse_leaf(screen_domain), self.name),
             attr_domain)
 
-    def get_state_attrs(self, record):
-        result = super(M2OField, self).get_state_attrs(record)
-        if (record.parent_name == self.name
-                and self.attrs['relation'] == record.group.parent.model_name):
-            result = result.copy()
-            result['readonly'] = True
-        return result
+    def get_on_change_value(self, record):
+        if record.parent_name == self.name and record.parent:
+            return record.parent.get_on_change_value(
+                skip={record.group.child_name})
+        return super(M2OField, self).get_on_change_value(record)
 
 
 class O2OField(M2OField):
@@ -469,7 +446,6 @@ class O2MField(Field):
     def __init__(self, attrs):
         super(O2MField, self).__init__(attrs)
         self.in_on_change = False
-        self.context = {}
 
     def sig_changed(self, record):
         if not self.in_on_change:
@@ -504,11 +480,12 @@ class O2MField(Field):
         from group import Group
         parent_name = self.attrs.get('relation_field', '')
         fields = fields or {}
+        context = record.expr_eval(self.attrs.get('context', {}))
         group = Group(self.attrs['relation'], fields,
                 parent=record,
                 parent_name=parent_name,
                 child_name=self.name,
-                context=self.context,
+                context=context,
                 parent_datetime_field=self.attrs.get('datetime_field'))
         if not fields and record.model_name == self.attrs['relation']:
             group.fields = record.group.fields
@@ -586,7 +563,8 @@ class O2MField(Field):
         for record2 in record.value[self.name]:
             if not (record2.deleted or record2.removed):
                 result.append(
-                    record2.get_on_change_value())
+                    record2.get_on_change_value(
+                        skip={self.attrs.get('relation_field', '')}))
         return result
 
     def _set_value(self, record, value, default=False):
@@ -671,11 +649,12 @@ class O2MField(Field):
         record.modified_fields.setdefault(self.name)
 
     def set_on_change(self, record, value):
+        self._set_default_value(record)
         if isinstance(value, (list, tuple)):
             self._set_value(record, value)
             record.modified_fields.setdefault(self.name)
             record.signal('record-modified')
-            return True
+            return
 
         if value and (value.get('add') or value.get('update')):
             context = self.context_get(record)
@@ -689,7 +668,7 @@ class O2MField(Field):
                     fields = RPCExecute('model', self.attrs['relation'],
                         'fields_get', list(field_names), context=context)
                 except RPCException:
-                    return False
+                    return
             else:
                 fields = {}
 
@@ -710,8 +689,10 @@ class O2MField(Field):
             record.value[self.name].add_fields(fields, signal=False)
             for index, vals in value.get('add', []):
                 new_record = record.value[self.name].new(default=False)
-                record.value[self.name].add(new_record, index)
+                record.value[self.name].add(new_record, index, signal=False)
                 new_record.set_on_change(vals)
+            if value.get('add'):
+                record.value[self.name].signal('group-changed', new_record)
 
             for vals in value.get('update', []):
                 if 'id' not in vals:
@@ -719,28 +700,32 @@ class O2MField(Field):
                 record2 = record.value[self.name].get(vals['id'])
                 if record2 is not None:
                     record2.set_on_change(vals)
-        return True
 
-    def validation_domains(self, record):
-        screen_domain, attr_domain = self.domains_get(record)
+    def validation_domains(self, record, pre_validate=None):
+        screen_domain, attr_domain = self.domains_get(record, pre_validate)
         return screen_domain
 
-    def validate(self, record, softvalidation=False):
+    def validate(self, record, softvalidation=False, pre_validate=None):
         if self.attrs.get('readonly'):
             return True
-        res = True
+        test = True
+        ldomain = localize_domain(domain_inversion(
+                record.group.clean4inversion(pre_validate or []), self.name,
+                EvalEnvironment(record)), self.name)
+        if isinstance(ldomain, bool):
+            if ldomain:
+                ldomain = []
+            else:
+                ldomain = [('id', '=', None)]
         for record2 in record.value.get(self.name, []):
             if not record2.loaded and record2.id >= 0:
                 continue
-            if not record2.validate(softvalidation=softvalidation):
-                if not record2.modified:
-                    record.value[self.name].remove(record2)
-                else:
-                    res = False
-        if not super(O2MField, self).validate(record, softvalidation):
-            res = False
-        self.get_state_attrs(record)['valid'] = res
-        return res
+            test &= record2.validate(softvalidation=softvalidation,
+                pre_validate=ldomain)
+        test &= super(O2MField, self).validate(record, softvalidation,
+            pre_validate)
+        self.get_state_attrs(record)['valid'] = test
+        return test
 
     def state_set(self, record, states=('readonly', 'required', 'invisible')):
         self._set_default_value(record)
@@ -776,12 +761,6 @@ class ReferenceField(Field):
             return None
 
     def get(self, record):
-        if record.parent_name == self.name:
-            if record.parent:
-                return '%s,%s' % (record.group.parent.model_name,
-                    record.parent.id)
-            else:
-                return None
         if (record.value.get(self.name)
                 and record.value[self.name][0]
                 and record.value[self.name][1] >= -1):
@@ -811,12 +790,6 @@ class ReferenceField(Field):
             force_change=force_change)
 
     def set(self, record, value):
-        if record.parent_name == self.name:
-            if record.parent:
-                value = '%s,%s' % (record.group.parent.model_name,
-                    record.parent.id)
-            else:
-                value = None
         if not value:
             record.value[self.name] = self._default
             return
@@ -847,6 +820,12 @@ class ReferenceField(Field):
         record.value[self.name] = ref_model, ref_id
         record.value[self.name + '.rec_name'] = rec_name
 
+    def get_on_change_value(self, record):
+        if record.parent_name == self.name and record.parent:
+            return record.parent.model_name, record.parent.get_on_change_value(
+                skip={record.group.child_name})
+        return super(ReferenceField, self).get_on_change_value(record)
+
 
 class BinaryField(Field):
 
@@ -905,11 +884,14 @@ class DictField(Field):
 
     _default = {}
 
+    def get(self, record):
+        return super(DictField, self).get(record) or self._default
+
     def get_client(self, record):
         return super(DictField, self).get_client(record) or self._default
 
-    def validation_domains(self, record):
-        screen_domain, attr_domain = self.domains_get(record)
+    def validation_domains(self, record, pre_validate=None):
+        screen_domain, attr_domain = self.domains_get(record, pre_validate)
         return screen_domain
 
     def domain_get(self, record):
