@@ -18,6 +18,7 @@ import logging
 
 from tryton.gui.window.view_form.model.group import Group
 from tryton.gui.window.view_form.view.screen_container import ScreenContainer
+from tryton.gui.window.view_form.view import View
 from tryton.signal_event import SignalEvent
 from tryton.config import CONFIG
 from tryton.exceptions import TrytonServerError, TrytonServerUnavailable
@@ -276,8 +277,8 @@ class Screen(SignalEvent):
             if hasattr(view, 'group_list_changed'):
                 view.group_list_changed(group, signal)
 
-    def _record_modified(self, group, signal, *args):
-        self.signal('record-modified')
+    def _record_modified(self, group, signal):
+        self.signal('record-modified', signal)
 
     def _group_changed(self, group, record):
         if not self.parent:
@@ -329,7 +330,7 @@ class Screen(SignalEvent):
 
     def default_row_activate(self):
         if (self.current_view.view_type == 'tree' and
-                self.current_view.widget_tree.keyword_open):
+                self.current_view.attributes.get('keyword_open')):
             return Action.exec_keyword('tree_open', {
                 'model': self.model_name,
                 'id': self.id_get(),
@@ -372,7 +373,8 @@ class Screen(SignalEvent):
                     break
         self.screen_container.set(self.current_view.widget)
         self.display()
-        self.set_cursor()
+        # Postpone set of the cursor to ensure widgets are allocated
+        gobject.idle_add(self.set_cursor)
 
     def load_view_to_load(self):
         if len(self.view_to_load):
@@ -402,14 +404,13 @@ class Screen(SignalEvent):
         view_id = view['view_id']
 
         xml_dom = xml.dom.minidom.parseString(arch)
-        for node in xml_dom.childNodes:
-            if node.localName == 'tree':
-                self.fields_view_tree = view
-            break
+        root, = xml_dom.childNodes
+        if root.tagName == 'tree':
+            self.fields_view_tree = view
 
         # Ensure that loading is always lazy for fields on form view
         # and always eager for fields on tree or graph view
-        if node.localName == 'form':
+        if root.tagName == 'form':
             loading = 'lazy'
         else:
             loading = 'eager'
@@ -419,18 +420,11 @@ class Screen(SignalEvent):
             else:
                 fields[field]['loading'] = \
                     self.group.fields[field].attrs['loading']
-
-        children_field = view.get('field_childs')
-
-        from tryton.gui.window.view_form.view.widget_parse import WidgetParse
         self.group.add_fields(fields)
-
-        parser = WidgetParse(parent=self.parent)
-        view = parser.parse(self, xml_dom, self.group.fields,
-                children_field=children_field)
+        view = View.parse(self, xml_dom, view.get('field_childs'))
         view.view_id = view_id
-
         self.views.append(view)
+
         return view
 
     def new(self, default=True):
@@ -451,14 +445,14 @@ class Screen(SignalEvent):
             previous_view.set_default_date(record, selected_date)
         self.current_record = record
         self.display()
-        self.set_cursor(new=True)
+        # Postpone set of the cursor to ensure widgets are allocated
+        gobject.idle_add(self.set_cursor, True)
         return self.current_record
 
     def new_model_position(self):
         position = -1
         if (self.current_view and self.current_view.view_type == 'tree'
-                and hasattr(self.current_view.widget_tree, 'editable')
-                and self.current_view.widget_tree.editable == 'top'):
+                and self.current_view.attributes.get('editable') == 'top'):
             position = 0
         return position
 
@@ -576,12 +570,13 @@ class Screen(SignalEvent):
 
         if delete:
             for record in records:
-                if record.parent:
-                    record.parent.save(force_reload=False)
                 if record in record.group.record_deleted:
                     record.group.record_deleted.remove(record)
                 if record in record.group.record_removed:
                     record.group.record_removed.remove(record)
+                if record.parent:
+                    # Save parent without deleted children
+                    record.parent.save(force_reload=False)
                 record.destroy()
 
         if idx > 0:
@@ -608,53 +603,81 @@ class Screen(SignalEvent):
 
     def set_tree_state(self):
         view = self.current_view
-        if (not CONFIG['client.save_tree_state']
-                or not view
-                or view.view_type != 'tree'
-                or not self.group):
+        if view.view_type not in ('tree', 'form'):
             return
         if id(view) in self.tree_states_done:
             return
+        if view.view_type == 'form' and self.tree_states_done:
+            return
         parent = self.parent.id if self.parent else None
+        expanded_nodes, selected_nodes = [], []
+        timestamp = self.parent._timestamp if self.parent else None
         state = self.tree_states[parent][view.children_field]
-        if state is None:
+        if state:
+            state_timestamp, expanded_nodes, selected_nodes = state
+            if (timestamp != state_timestamp
+                    and view.view_type != 'form'):
+                state = None
+        if state is None and CONFIG['client.save_tree_state']:
             json_domain = self.get_tree_domain(parent)
             try:
                 expanded_nodes, selected_nodes = RPCExecute('model',
                     'ir.ui.view_tree_state', 'get',
                     self.model_name, json_domain,
-                    self.current_view.children_field)
+                    view.children_field)
                 expanded_nodes = json.loads(expanded_nodes)
                 selected_nodes = json.loads(selected_nodes)
             except RPCException:
                 logging.getLogger(__name__).warn(
                     _('Unable to get view tree state'))
-                expanded_nodes = []
-                selected_nodes = []
             self.tree_states[parent][view.children_field] = (
-                expanded_nodes, selected_nodes)
+                timestamp, expanded_nodes, selected_nodes)
+        if view.view_type == 'tree':
+            view.expand_nodes(expanded_nodes)
+            view.select_nodes(selected_nodes)
         else:
-            expanded_nodes, selected_nodes = state
-        view.expand_nodes(expanded_nodes)
-        view.select_nodes(selected_nodes)
+            if selected_nodes:
+                record = None
+                for node in selected_nodes[0]:
+                    new_record = self.group.get(node)
+                    if node < 0 and -node < len(self.group):
+                        # Negative id is the index of the new record
+                        new_record = self.group[-node]
+                    if not new_record:
+                        break
+                    else:
+                        record = new_record
+                if record and record != self.current_record:
+                    self.current_record = record
+                    # Force a display of the view to synchronize the
+                    # widgets with the new record
+                    view.display()
         self.tree_states_done.add(id(view))
 
     def save_tree_state(self, store=True):
         if not CONFIG['client.save_tree_state']:
             return
+        parent = self.parent.id if self.parent else None
+        timestamp = self.parent._timestamp if self.parent else None
         for view in self.views:
             if view.view_type == 'form':
                 for widgets in view.widgets.itervalues():
                     for widget in widgets:
                         if hasattr(widget, 'screen'):
                             widget.screen.save_tree_state(store)
-            elif (view.view_type == 'tree' and view.children_field):
-                parent = self.parent.id if self.parent else None
+                if len(self.views) == 1 and self.current_record:
+                    path = self.current_record.id
+                    if path < 0:
+                        path = -self.current_record.group.index(
+                            self.current_record)
+                    self.tree_states[parent][view.children_field] = (
+                        timestamp, [], [[path]])
+            elif view.view_type == 'tree':
                 paths = view.get_expanded_paths()
                 selected_paths = view.get_selected_paths()
                 self.tree_states[parent][view.children_field] = (
-                    paths, selected_paths)
-                if store:
+                    timestamp, paths, selected_paths)
+                if store and view.attributes.get('tree_state', False):
                     json_domain = self.get_tree_domain(parent)
                     json_paths = json.dumps(paths)
                     json_selected_path = json.dumps(selected_paths)
@@ -680,7 +703,7 @@ class Screen(SignalEvent):
         self.tree_states_done.clear()
         self.group.load(ids, modified=modified)
         self.current_view.reset()
-        if ids:
+        if ids and self.current_view.view_type != 'calendar':
             self.display(ids[0])
         else:
             self.current_record = None
@@ -695,7 +718,7 @@ class Screen(SignalEvent):
             if (self.current_record
                     and self.current_record in self.current_record.group):
                 pass
-            elif self.group:
+            elif self.group and self.current_view.view_type != 'calendar':
                 self.current_record = self.group[0]
             else:
                 self.current_record = None
@@ -719,12 +742,14 @@ class Screen(SignalEvent):
         view.set_value()
         self.set_cursor(reset_view=False)
         if view.view_type == 'tree' and len(self.group):
-            start, end = view.widget_tree.get_visible_range()
-            vadjustment = view.widget_tree.get_vadjustment()
-            vadjustment.value = vadjustment.value + vadjustment.page_increment
-            store = view.store
-            iter_ = store.get_iter(end)
-            self.current_record = store.get_value(iter_, 0)
+            start, end = view.treeview.get_visible_range()
+            vadjustment = view.treeview.get_vadjustment()
+            vadjustment.value = min(
+                vadjustment.value + vadjustment.page_increment,
+                vadjustment.get_upper())
+            model = view.treeview.get_model()
+            iter_ = model.get_iter(end)
+            self.current_record = model.get_value(iter_, 0)
         elif (view.view_type == 'form'
                 and self.current_record
                 and self.current_record.group):
@@ -755,7 +780,7 @@ class Screen(SignalEvent):
             self.current_record = record
         elif view.view_type == 'calendar':
             record = self.current_record
-            goocalendar = view.children['goocalendar']
+            goocalendar = view.widgets['goocalendar']
             date = goocalendar.selected_date
             year = date.year
             month = date.month
@@ -784,12 +809,14 @@ class Screen(SignalEvent):
         view.set_value()
         self.set_cursor(reset_view=False)
         if view.view_type == 'tree' and len(self.group):
-            start, end = view.widget_tree.get_visible_range()
-            vadjustment = view.widget_tree.get_vadjustment()
-            vadjustment.value = vadjustment.value - vadjustment.page_increment
-            store = view.store
-            iter_ = store.get_iter(start)
-            self.current_record = store.get_value(iter_, 0)
+            start, end = view.treeview.get_visible_range()
+            vadjustment = view.treeview.get_vadjustment()
+            vadjustment.value = min(
+                vadjustment.value - vadjustment.page_increment,
+                vadjustment.get_lower())
+            model = view.treeview.get_model()
+            iter_ = model.get_iter(start)
+            self.current_record = model.get_value(iter_, 0)
         elif (view.view_type == 'form'
                 and self.current_record
                 and self.current_record.group):
@@ -810,7 +837,7 @@ class Screen(SignalEvent):
             self.current_record = record
         elif view.view_type == 'calendar':
             record = self.current_record
-            goocalendar = view.children['goocalendar']
+            goocalendar = view.widgets['goocalendar']
             date = goocalendar.selected_date
             year = date.year
             month = date.month
@@ -874,12 +901,22 @@ class Screen(SignalEvent):
         return buttons
 
     def button(self, button):
-        'Execute button on the current record'
+        'Execute button on the selected records'
         if button.get('confirm', False) and not sur(button['confirm']):
             return
-        record = self.current_record
-        if not record.save(force_reload=False):
+        self.current_view.set_value()
+        if not self.current_record.save(force_reload=False):
             return
+        fields = self.current_view.get_fields()
+        for record in self.selected_records:
+            domain = record.expr_eval(
+                button.get('states', {})).get('pre_validate', [])
+            if not record.validate(fields, pre_validate=domain):
+                self.display(set_cursor=True)
+                if domain:
+                    # Reset valid state with normal domain
+                    record.validate(fields)
+                return
         ids = [r.id for r in self.selected_records]
         try:
             action = RPCExecute('model', self.model_name, button['name'],
