@@ -3,24 +3,25 @@
 import gtk
 import gettext
 
-from interface import WidgetInterface
+from .widget import Widget
 from tryton.common.date_widget import DateEntry
 from tryton.translate import date_format
 
 _ = gettext.gettext
 
 
-class Calendar(WidgetInterface):
+class Calendar(Widget):
     "Calendar"
 
-    def __init__(self, field_name, model_name, attrs=None):
-        super(Calendar, self).__init__(field_name, model_name, attrs=attrs)
+    def __init__(self, view, attrs):
+        super(Calendar, self).__init__(view, attrs)
 
         self.widget = gtk.HBox()
         self.entry = DateEntry('')
         self.entry.set_property('activates_default', True)
         self.entry.connect('key_press_event', self.sig_key_press)
         self.entry.connect('activate', self.sig_activate)
+        self.entry.connect('changed', lambda _: self.send_modified())
         self.entry.connect('focus-out-event', lambda x, y: self._focus_out())
         self.widget.pack_start(self.entry, expand=False, fill=False)
 
@@ -44,9 +45,6 @@ class Calendar(WidgetInterface):
     def sig_key_press(self, widget, event):
         self.send_modified()
 
-    def grab_focus(self):
-        return self.entry.grab_focus()
-
     def set_value(self, record, field):
         field.set_client(record, self.get_value())
 
@@ -76,9 +74,6 @@ class Calendar(WidgetInterface):
 class DateTime(Calendar):
     "DateTime"
 
-    def __init__(self, field_name, model_name, attrs=None):
-        super(DateTime, self).__init__(field_name, model_name, attrs=attrs)
-
     def get_format(self, record, field):
         return date_format() + ' ' + field.time_format(record)
 
@@ -86,8 +81,8 @@ class DateTime(Calendar):
 class Time(Calendar):
     "Time"
 
-    def __init__(self, field_name, model_name, attrs=None):
-        super(Time, self).__init__(field_name, model_name, attrs=attrs)
+    def __init__(self, view, attrs):
+        super(Time, self).__init__(view, attrs)
         self.entry.set_icon_from_stock(gtk.ENTRY_ICON_SECONDARY, None)
 
     def get_format(self, record, field):
