@@ -1,26 +1,21 @@
 #This file is part of Tryton.  The COPYRIGHT file at the top level of
 #this repository contains the full copyright notices and license terms.
-from __future__ import with_statement
 import gtk
 import gettext
 import os
 import tempfile
-from base64 import encodestring, decodestring
-from tryton.common import file_selection, Tooltips, file_open
-from tryton.config import PIXMAPS_DIR
-from interface import WidgetInterface
+from tryton.common import (file_selection, Tooltips, file_open, slugify,
+    resize_pixbuf, data2pixbuf, BIG_IMAGE_SIZE)
+from .widget import Widget
 import urllib
 
 _ = gettext.gettext
 
-NOIMAGE = open(os.path.join(PIXMAPS_DIR, 'tryton-noimage.png'), 'rb').read()
 
+class Image(Widget):
 
-class Image(WidgetInterface):
-
-    def __init__(self, field_name, model_name, window, attrs=None):
-        super(Image, self).__init__(field_name, model_name, window,
-                attrs=attrs)
+    def __init__(self, view, attrs):
+        super(Image, self).__init__(view, attrs)
 
         self.filename = attrs.get('filename')
         self.height = int(attrs.get('img_height', 100))
@@ -56,7 +51,8 @@ class Image(WidgetInterface):
             if self.filename:
                 self.but_open = gtk.Button()
                 img_open = gtk.Image()
-                img_open.set_from_stock('tryton-open', gtk.ICON_SIZE_SMALL_TOOLBAR)
+                img_open.set_from_stock('tryton-open',
+                    gtk.ICON_SIZE_SMALL_TOOLBAR)
                 self.but_open.set_image(img_open)
                 self.but_open.set_relief(gtk.RELIEF_NONE)
                 self.but_open.connect('clicked', self.sig_open)
@@ -67,7 +63,8 @@ class Image(WidgetInterface):
 
             self.but_save_as = gtk.Button()
             img_save_as = gtk.Image()
-            img_save_as.set_from_stock('tryton-save', gtk.ICON_SIZE_SMALL_TOOLBAR)
+            img_save_as.set_from_stock('tryton-save',
+                gtk.ICON_SIZE_SMALL_TOOLBAR)
             self.but_save_as.set_image(img_save_as)
             self.but_save_as.set_relief(gtk.RELIEF_NONE)
             self.but_save_as.connect('clicked', self.sig_save_as)
@@ -76,7 +73,8 @@ class Image(WidgetInterface):
 
             self.but_remove = gtk.Button()
             img_remove = gtk.Image()
-            img_remove.set_from_stock('tryton-clear', gtk.ICON_SIZE_SMALL_TOOLBAR)
+            img_remove.set_from_stock('tryton-clear',
+                gtk.ICON_SIZE_SMALL_TOOLBAR)
             self.but_remove.set_image(img_remove)
             self.but_remove.set_relief(gtk.RELIEF_NONE)
             self.but_remove.connect('clicked', self.sig_remove)
@@ -101,9 +99,6 @@ class Image(WidgetInterface):
     def filename_field(self):
         return self.record.group.fields.get(self.filename)
 
-    def grab_focus(self):
-        return self.image.grab_focus()
-
     def _readonly_set(self, value):
         self._readonly = value
         if self.but_add:
@@ -127,11 +122,10 @@ class Image(WidgetInterface):
         for pat in ("*.png", "*.jpg", "*.gif", "*.tif", "*.xpm"):
             filter_image.add_pattern(pat)
 
-        filename = file_selection(_('Open...'), parent=self.window,
-                preview=True, filters=[filter_image, filter_all])
+        filename = file_selection(_('Open...'), preview=True,
+            filters=[filter_image, filter_all])
         if filename:
-            self.field.set_client(self.record,
-                    encodestring(open(filename, 'rb').read()))
+            self.field.set_client(self.record, open(filename, 'rb').read())
             if self.filename_field:
                 self.filename_field.set_client(self.record,
                         os.path.basename(filename))
@@ -141,25 +135,28 @@ class Image(WidgetInterface):
         if not self.filename_field:
             return
         dtemp = tempfile.mkdtemp(prefix='tryton_')
-        filename = self.filename_field.get(self.record).replace(
-                os.sep, '_').replace(os.altsep or os.sep, '_')
+        filename = self.filename_field.get(self.record)
+        if not filename:
+            return
+        root, ext = os.path.splitext(filename)
+        filename = ''.join([slugify(root), os.extsep, slugify(ext)])
         file_path = os.path.join(dtemp, filename)
         with open(file_path, 'wb') as fp:
-            fp.write(decodestring(self.field.get(self.record)))
+            fp.write(self.field.get_data(self.record))
         root, type_ = os.path.splitext(filename)
         if type_:
             type_ = type_[1:]
-        file_open(file_path, type_, self.window)
+        file_open(file_path, type_)
 
     def sig_save_as(self, widget):
         filename = ''
         if self.filename_field:
             filename = self.filename_field.get(self.record)
         filename = file_selection(_('Save As...'), filename=filename,
-                parent=self.window, action=gtk.FILE_CHOOSER_ACTION_SAVE)
+            action=gtk.FILE_CHOOSER_ACTION_SAVE)
         if filename:
             with open(filename, 'wb') as fp:
-                fp.write(decodestring(self.field.get(self.record)))
+                fp.write(self.field.get_data(self.record))
 
     def sig_remove(self, widget):
         self.field.set_client(self.record, False)
@@ -178,68 +175,30 @@ class Image(WidgetInterface):
         if info == 0:
             uri = selection.get_text().split('\n')[0]
             if uri:
-                self.field.set_client(self.record,
-                        encodestring(urllib.urlopen(uri).read()))
+                self.field.set_client(self.record, urllib.urlopen(uri).read())
             self.update_img()
         elif info == 1:
             uri = selection.data.split('\r\n')[0]
             if uri:
-                self.field.set_client(self.record,
-                        encodestring(urllib.urlopen(uri).read()))
+                self.field.set_client(self.record, urllib.urlopen(uri).read())
             self.update_img()
         elif info == 2:
             data = selection.get_pixbuf()
             if data:
-                self.field.set_client(self.record,
-                        encodestring(data))
+                self.field.set_client(self.record, data)
                 self.update_img()
 
     def update_img(self):
         value = None
         if self.field:
             value = self.field.get_client(self.record)
-        if not value:
-            data = NOIMAGE
-        else:
-            data = decodestring(value)
-
-        pixbuf = None
-        for ftype in ('jpeg', 'gif', 'png', 'bmp', 'svg'):
-            try:
-                loader = gtk.gdk.PixbufLoader(ftype)
-                loader.write(data, len(data))
-                pixbuf = loader.get_pixbuf()
-                loader.close()
-            except Exception:
-                continue
-            if pixbuf:
-                break
-        if not pixbuf:
-            loader = gtk.gdk.PixbufLoader('png')
-            loader.write(NOIMAGE, len(NOIMAGE))
-            pixbuf = loader.get_pixbuf()
-            loader.close()
-
-        img_height = pixbuf.get_height()
-        if img_height > self.height:
-            height = self.height
-        else:
-            height = img_height
-
-        img_width = pixbuf.get_width()
-        if img_width > self.width:
-            width = self.width
-        else:
-            width = img_width
-
-        if (img_width / width) < (img_height / height):
-            width = float(img_width) / float(img_height) * float(height)
-        else:
-            height = float(img_height) / float(img_width) * float(width)
-
-        scaled = pixbuf.scale_simple(int(width), int(height),
-                gtk.gdk.INTERP_BILINEAR)
-        self.image.set_from_pixbuf(scaled)
+        if isinstance(value, (int, long)):
+            if value > BIG_IMAGE_SIZE:
+                value = False
+            else:
+                value = self.field.get_data(self.record)
+        pixbuf = resize_pixbuf(data2pixbuf(value), self.width, self.height)
+        self.image.set_from_pixbuf(pixbuf)
 
     def display(self, record, field):
         if not field:
