@@ -3,24 +3,23 @@
 import gtk
 import gobject
 import gettext
-from interface import WidgetInterface
+
+from .widget import Widget
 import tryton.common as common
 from tryton.gui.window.view_form.screen import Screen
 from tryton.gui.window.win_search import WinSearch
 from tryton.gui.window.win_form import WinForm
-from tryton.config import CONFIG
 from tryton.common.popup_menu import populate
-from tryton.common import RPCExecute, RPCException
 from tryton.common.completion import get_completion, update_completion
 from tryton.common.entry_position import manage_entry_position
 
 _ = gettext.gettext
 
 
-class Many2One(WidgetInterface):
+class Many2One(Widget):
 
-    def __init__(self, field_name, model_name, attrs=None):
-        super(Many2One, self).__init__(field_name, model_name, attrs=attrs)
+    def __init__(self, view, attrs):
+        super(Many2One, self).__init__(view, attrs)
 
         self.widget = gtk.HBox(spacing=0)
         self.widget.set_property('sensitive', True)
@@ -49,38 +48,15 @@ class Many2One(WidgetInterface):
         else:
             self.wid_completion = None
 
-        self.but_open = gtk.Button()
-        img_find = gtk.Image()
-        img_find.set_from_stock('tryton-find', gtk.ICON_SIZE_SMALL_TOOLBAR)
-        self.but_open.set_image(img_find)
-        self.but_open.set_relief(gtk.RELIEF_NONE)
-        self.but_open.connect('clicked', self.sig_edit)
-        self.but_open.set_alignment(0.5, 0.5)
-
-        self.but_new = gtk.Button()
-        img_new = gtk.Image()
-        img_new.set_from_stock('tryton-new', gtk.ICON_SIZE_SMALL_TOOLBAR)
-        self.but_new.set_image(img_new)
-        self.but_new.set_relief(gtk.RELIEF_NONE)
-        self.but_new.connect('clicked', self.sig_new)
-        self.but_new.set_alignment(0.5, 0.5)
-
-        self.widget.pack_end(self.but_new, expand=False, fill=False)
-        self.widget.pack_end(self.but_open, expand=False, fill=False)
-        self.widget.pack_end(self.wid_text, expand=True, fill=True)
+        self.wid_text.set_icon_from_stock(gtk.ENTRY_ICON_SECONDARY,
+            'tryton-find')
+        self.wid_text.connect('icon-press', self.sig_edit)
 
+        self.widget.pack_start(self.wid_text, expand=True, fill=True)
         self.widget.set_focus_chain([self.wid_text])
 
-        self.tooltips = common.Tooltips()
-        self.tooltips.set_tip(self.but_new, _('Create a new record <F3>'))
-        self.tooltips.set_tip(self.but_open, _('Open a record <F2>'))
-        self.tooltips.enable()
-
         self._readonly = False
 
-    def grab_focus(self):
-        return self.wid_text.grab_focus()
-
     def get_model(self):
         return self.attrs['relation']
 
@@ -93,21 +69,24 @@ class Many2One(WidgetInterface):
             self.widget.set_focus_chain([self.wid_text])
 
     def _set_button_sensitive(self):
+        self.wid_text.set_editable(not self._readonly)
+        self.wid_text.set_icon_sensitive(gtk.ENTRY_ICON_SECONDARY,
+            self.read_access)
+
+    def get_access(self, type_):
         model = self.get_model()
         if model:
-            access = common.MODELACCESS[model]
+            return common.MODELACCESS[model][type_]
         else:
-            access = {
-                'create': True,
-                'read': True,
-                }
-        self.wid_text.set_editable(not self._readonly)
-        self.but_new.set_sensitive(bool(
-                not self._readonly
-                and self.attrs.get('create', True)
-                and access['create']))
-        self.but_open.set_sensitive(bool(
-                access['read']))
+            return True
+
+    @property
+    def read_access(self):
+        return self.get_access('read')
+
+    @property
+    def create_access(self):
+        return self.attrs.get('create', True) and self.get_access('create')
 
     @property
     def modified(self):
@@ -151,27 +130,7 @@ class Many2One(WidgetInterface):
                             self.record)['required'])):
                 domain = self.field.domain_get(self.record)
                 context = self.field.context_get(self.record)
-                self.wid_text.grab_focus()
-
-                try:
-                    if self.wid_text.get_text():
-                        dom = [('rec_name', 'ilike',
-                            '%' + self.wid_text.get_text() + '%'),
-                            domain]
-                    else:
-                        dom = domain
-                    ids = RPCExecute('model', model, 'search', dom, 0,
-                        CONFIG['client.limit'], None, context=context)
-                except RPCException:
-                    self.focus_out = True
-                    self.changed = True
-                    return
-                if len(ids) == 1:
-                    self.field.set_client(self.record,
-                        self.value_from_id(ids[0]), force_change=True)
-                    self.focus_out = True
-                    self.changed = True
-                    return
+                text = self.wid_text.get_text().decode('utf-8')
 
                 def callback(result):
                     if result:
@@ -182,11 +141,12 @@ class Many2One(WidgetInterface):
                     self.focus_out = True
                     self.changed = True
 
-                WinSearch(model, callback, sel_multi=False,
-                    ids=ids, context=context, domain=domain,
+                win = WinSearch(model, callback, sel_multi=False,
+                    context=context, domain=domain,
                     view_ids=self.attrs.get('view_ids', '').split(','),
                     views_preload=self.attrs.get('views', {}),
-                    new=self.but_new.get_property('sensitive'))
+                    new=self.create_access)
+                win.screen.search_filter(text)
                 return
         self.focus_out = True
         self.changed = True
@@ -243,26 +203,7 @@ class Many2One(WidgetInterface):
         elif model and not self._readonly:
             domain = self.field.domain_get(self.record)
             context = self.field.context_get(self.record)
-            self.wid_text.grab_focus()
-
-            try:
-                if self.wid_text.get_text():
-                    dom = [('rec_name', 'ilike',
-                        '%' + self.wid_text.get_text() + '%'),
-                        domain]
-                else:
-                    dom = domain
-                ids = RPCExecute('model', model, 'search', dom, 0,
-                    CONFIG['client.limit'], None, context=context)
-            except RPCException:
-                self.focus_out = True
-                self.changed = True
-                return False
-            if len(ids) == 1:
-                self.field.set_client(self.record, self.value_from_id(ids[0]),
-                    force_change=True)
-                self.focus_out = True
-                return True
+            text = self.wid_text.get_text().decode('utf-8')
 
             def callback(result):
                 if result:
@@ -270,11 +211,12 @@ class Many2One(WidgetInterface):
                         self.value_from_id(*result[0]), force_change=True)
                 self.focus_out = True
                 self.changed = True
-            WinSearch(model, callback, sel_multi=False,
-                ids=ids, context=context, domain=domain,
+            win = WinSearch(model, callback, sel_multi=False,
+                context=context, domain=domain,
                 view_ids=self.attrs.get('view_ids', '').split(','),
                 views_preload=self.attrs.get('views', {}),
-                new=self.but_new.get_property('sensitive'))
+                new=self.create_access)
+            win.screen.search_filter(text)
             return
         self.focus_out = True
         self.changed = True
@@ -287,11 +229,10 @@ class Many2One(WidgetInterface):
             activate_keys.append(gtk.keysyms.Return)
         if (event.keyval == gtk.keysyms.F3
                 and editable
-                and self.but_new.get_property('sensitive')):
+                and self.create_access):
             self.sig_new(widget, event)
             return True
-        elif (event.keyval == gtk.keysyms.F2
-                and self.but_open.get_property('sensitive')):
+        elif event.keyval == gtk.keysyms.F2 and self.read_access:
             self.sig_edit(widget)
             return True
         elif (event.keyval in activate_keys
@@ -316,9 +257,12 @@ class Many2One(WidgetInterface):
                 position = self.wid_text.get_position()
                 self.field.set_client(self.record,
                     self.value_from_id(None, ''))
-                # Restore text and position after display
-                self.wid_text.set_text(text)
-                self.wid_text.set_position(position)
+                # The value of the field could be different of None
+                # in such case, the original text should not be restored
+                if not self.wid_text.get_text():
+                    # Restore text and position after display
+                    self.wid_text.set_text(text)
+                    self.wid_text.set_position(position)
             gobject.idle_add(clean)
         return False
 
@@ -347,16 +291,13 @@ class Many2One(WidgetInterface):
             self.wid_text.set_position(0)
             self.changed = True
             return False
-        img = self.but_open.get_image()
-        current_stock = img.get_stock()[0]
         self.set_text(field.get_client(record))
-        value = field.get(record)
-        if self.has_target(value) and current_stock != 'tryton-open':
-            img.set_from_stock('tryton-open', gtk.ICON_SIZE_SMALL_TOOLBAR)
-            self.tooltips.set_tip(self.but_open, _('Open a record <F2>'))
-        elif not self.has_target(value) and current_stock != 'tryton-find':
-            img.set_from_stock('tryton-find', gtk.ICON_SIZE_SMALL_TOOLBAR)
-            self.tooltips.set_tip(self.but_open, _('Search a record <F2>'))
+        if self.has_target(field.get(record)):
+            stock, tooltip = 'tryton-open', _('Open a record <F2>')
+        else:
+            stock, tooltip = 'tryton-find', _('Search a record <F2>')
+        self.wid_text.set_icon_from_stock(gtk.ENTRY_ICON_SECONDARY, stock)
+        self.wid_text.set_icon_tooltip_text(gtk.ENTRY_ICON_SECONDARY, tooltip)
         self.changed = True
 
     def _populate_popup(self, widget, menu):
