@@ -1,11 +1,11 @@
 #This file is part of Tryton.  The COPYRIGHT file at the top level of
 #this repository contains the full copyright notices and license terms.
 import gtk
-from interface import WidgetInterface
+from .widget import Widget
 import locale
 
 
-class ProgressBar(WidgetInterface):
+class ProgressBar(Widget):
     'Progress Bar'
     orientations = {
         'left_to_right': gtk.PROGRESS_LEFT_TO_RIGHT,
@@ -14,9 +14,8 @@ class ProgressBar(WidgetInterface):
         'top_to_bottom': gtk.PROGRESS_TOP_TO_BOTTOM,
     }
 
-    def __init__(self, field_name, model_name, window, attrs=None):
-        super(ProgressBar, self).__init__(field_name, model_name, window,
-                attrs=attrs)
+    def __init__(self, view, attrs):
+        super(ProgressBar, self).__init__(view, attrs)
         self.widget = gtk.ProgressBar()
         orientation = self.orientations.get(attrs.get('orientation',
             'left_to_right'), gtk.PROGRESS_LEFT_TO_RIGHT)
@@ -29,12 +28,6 @@ class ProgressBar(WidgetInterface):
             self.widget.set_fraction(0.0)
             return False
         value = float(field.get(record) or 0.0)
-        digits = field.attrs.get('digits', (16, 2))
-        if isinstance(self.digits, str):
-            digits = record.expr_eval(self.digits)
-        self.widget.set_text(locale.format('%.' + str(digits[1]) + 'f',
-            value, True))
+        digits = field.digits(record)
+        self.widget.set_text(locale.format('%.*f', (digits[1], value), True))
         self.widget.set_fraction(value / 100.0)
-
-    def display_value(self):
-        return self.widget.get_text()
