@@ -1,21 +1,21 @@
 #This file is part of Tryton.  The COPYRIGHT file at the top level of
 #this repository contains the full copyright notices and license terms.
-#This code is inspired by the pycha project (http://www.lorenzogil.com/projects/pycha/)
+#This code is inspired by the pycha project
+#(http://www.lorenzogil.com/projects/pycha/)
 import gtk
 from functools import reduce
-from tryton.common import hex2rgb, generateColorscheme, DT_FORMAT, \
-        DHM_FORMAT, COLOR_SCHEMES, datetime_strftime
+from tryton.common import hex2rgb, generateColorscheme, \
+        COLOR_SCHEMES, datetime_strftime
 from tryton.pyson import PYSONDecoder
 import locale
 import math
 import datetime
-import time
 from dateutil.relativedelta import relativedelta
 import tryton.rpc as rpc
 import cairo
 from tryton.action import Action
-from tryton.translate import date_format
 from tryton.gui.window import Window
+from tryton.translate import date_format
 
 
 class Popup(object):
@@ -65,13 +65,15 @@ class Popup(object):
     def enter(self, widget, event):
         self.win.hide()
 
+
 class Graph(gtk.DrawingArea):
     'Graph'
 
-    __gsignals__ = { "expose-event": "override" }
+    __gsignals__ = {"expose-event": "override"}
 
-    def __init__(self, xfield, yfields, attrs, model):
+    def __init__(self, view, xfield, yfields):
         super(Graph, self).__init__()
+        self.view = view
         self.xfield = xfield
         self.yfields = yfields
         self.datas = {}
@@ -83,8 +85,14 @@ class Graph(gtk.DrawingArea):
         self.connect('motion-notify-event', self.motion)
         self.connect('leave-notify-event', self.leave)
         self.popup = Popup(self)
-        self.attrs = attrs
-        self.model = model
+
+    @property
+    def attrs(self):
+        return self.view.attributes
+
+    @property
+    def model(self):
+        return self.view.screen.model_name
 
     def destroy(self):
         self.popup.destroy()
@@ -167,13 +175,12 @@ class Graph(gtk.DrawingArea):
     def drawGraph(self, cr, width, height):
         pass
 
-
     def YLabels(self):
         ylabels = []
         if self.yrange == 0.0:
             base = 1
         else:
-            base = 10**int(math.log(self.yrange, 10))
+            base = 10 ** int(math.log(self.yrange, 10))
         for i in xrange(int(self.yrange / base) + 1):
             val = int(self.minyval / base) * base + i * base
             h = (val - self.minyval) * self.yscale
@@ -276,7 +283,7 @@ class Graph(gtk.DrawingArea):
         cr.restore()
 
     def drawLegend(self, cr, width, height):
-        if not self.attrs.get('legend', True):
+        if not int(self.attrs.get('legend', 1)):
             return
 
         padding = 4
@@ -327,7 +334,7 @@ class Graph(gtk.DrawingArea):
 
     def _getLegendPosition(self, width, height):
         return self.area.x + self.area.w * 0.05, \
-                self.area.y + self.area.h * 0.05
+            self.area.y + self.area.h * 0.05
 
     def display(self, group):
         self.updateDatas(group)
@@ -356,7 +363,6 @@ class Graph(gtk.DrawingArea):
             self.ids[x].append(model.id)
             self.datas.setdefault(x, {})
             for yfield in self.yfields:
-                name = yfield['name']
                 key = yfield.get('key', yfield['name'])
                 self.datas[x].setdefault(key, 0.0)
                 if yfield.get('domain'):
@@ -364,19 +370,18 @@ class Graph(gtk.DrawingArea):
                     context['context'] = context.copy()
                     context['_user'] = rpc._USER
                     for field in model.group.fields:
-                        context[field] = model[field].get(model,
-                            check_load=False)
+                        context[field] = model[field].get(model)
                     if not PYSONDecoder(context).decode(yfield['domain']):
                         continue
                 if yfield['name'] == '#':
                     self.datas[x][key] += 1
                 else:
                     self.datas[x][key] += \
-                            float(model[yfield['name']].get(model))
+                        float(model[yfield['name']].get(model) or 0)
         if isinstance(minx, datetime.datetime):
             date = minx
             while date <= maxx:
-                self.labels[date] = datetime_strftime(date, DT_FORMAT)
+                self.labels[date] = datetime_strftime(date, date_format())
                 self.datas.setdefault(date, {})
                 for yfield in self.yfields:
                     self.datas[date].setdefault(
@@ -385,7 +390,7 @@ class Graph(gtk.DrawingArea):
         elif isinstance(minx, datetime.date):
             date = minx
             while date <= maxx:
-                self.labels[date] = datetime_strftime(date, DT_FORMAT)
+                self.labels[date] = datetime_strftime(date, date_format())
                 self.datas.setdefault(date, {})
                 for yfield in self.yfields:
                     self.datas[date].setdefault(
@@ -411,7 +416,8 @@ class Graph(gtk.DrawingArea):
             yLabelWidth = 0
         width = width - self.leftPadding - yLabelWidth - self.rightPadding
         height = height - self.topPadding - self.bottomPadding - xLabelHeight
-        self.area = Area(self.leftPadding + yLabelWidth, self.topPadding, width, height)
+        self.area = Area(self.leftPadding + yLabelWidth, self.topPadding,
+            width, height)
 
     def updateXY(self):
         self.maxxval = len(self.datas)
@@ -427,9 +433,9 @@ class Graph(gtk.DrawingArea):
             self.maxyval = 0.0
             self.minyval = 0.0
         else:
-            self.maxyval = max([reduce(lambda x, y: max(x, y), x.values()) \
+            self.maxyval = max([reduce(lambda x, y: max(x, y), x.values())
                     for x in self.datas.values()])
-            self.minyval = min([reduce(lambda x, y: min(x, y), x.values()) \
+            self.minyval = min([reduce(lambda x, y: min(x, y), x.values())
                     for x in self.datas.values()])
         if self.minyval > 0:
             self.minyval = 0.0
@@ -458,6 +464,7 @@ class Graph(gtk.DrawingArea):
     def _getDatasKeys(self):
         return [x.get('key', x['name']) for x in self.yfields]
 
+
 class Area(object):
 
     def __init__(self, x, y, w, h):
