@@ -8,18 +8,39 @@ try:
 except ImportError:
     import json
 import locale
-from interface import ParserView
 import gettext
+from functools import wraps
+
 from tryton.config import CONFIG
 from tryton.common.cellrendererbutton import CellRendererButton
 from tryton.common.cellrenderertoggle import CellRendererToggle
 from tryton.gui.window import Window
 from tryton.common.popup_menu import populate
-from tryton.common import RPCExecute, RPCException
+from tryton.common import RPCExecute, RPCException, node_attributes, Tooltips
+from . import View
+from .list_gtk.editabletree import EditableTreeView, TreeView
+from .list_gtk.widget import (Affix, Char, Int, Boolean, URL, Date, Datetime,
+    Time, Float, FloatTime, Binary, M2O, O2O, O2M, M2M, Selection, Reference,
+    ProgressBar, Button, Image)
 
 _ = gettext.gettext
 
 
+def delay(func):
+    """Decorator for ViewTree method to delay execution when idle and if
+    display counter did not change"""
+    @wraps(func)
+    def wrapper(self, *args, **kwargs):
+        def wait():
+            if not self.treeview.props.window:
+                return
+            if self.treeview.display_counter == display_counter:
+                func(self, *args, **kwargs)
+        display_counter = self.treeview.display_counter
+        gobject.idle_add(wait)
+    return wrapper
+
+
 def path_convert_id2pos(model, id_path):
     "This function will transform a path of id into a path of position"
     group = model.group
@@ -238,90 +259,344 @@ class AdaptModelGroup(gtk.GenericTreeModel):
         return record.parent
 
 
-class ViewList(ParserView):
+class ViewTree(View):
 
-    def __init__(self, screen, widget, children=None, state_widgets=None,
-            notebooks=None, cursor_widget=None, children_field=None):
-        super(ViewList, self).__init__(screen, widget, children, state_widgets,
-            notebooks, cursor_widget, children_field)
-        self.store = None
+    def __init__(self, screen, xml, children_field):
+        super(ViewTree, self).__init__(screen, xml)
         self.view_type = 'tree'
+        self.widgets = {}
+        self.state_widgets = []
+        self.children_field = children_field
+        self.sum_widgets = []
+        self.sum_box = gtk.HBox()
+        self.reload = False
+        if self.attributes.get('editable'):
+            self.treeview = EditableTreeView(self.attributes['editable'], self)
+        else:
+            self.treeview = TreeView()
 
-        vbox = gtk.VBox()
+        self.parse(xml)
+
+        self.treeview.set_property('rules-hint', True)
+        self.treeview.set_fixed_height_mode(True)
+        self.treeview.connect('button-press-event', self.__button_press)
+        self.treeview.connect('key-press-event', self.on_keypress)
+        self.treeview.connect_after('row-activated', self.__sig_switch)
+        if self.children_field:
+            self.treeview.connect('test-expand-row', self.test_expand_row)
+            self.treeview.set_expander_column(self.treeview.get_column(0))
+        self.treeview.set_rubber_banding(True)
+
+        selection = self.treeview.get_selection()
+        selection.set_mode(gtk.SELECTION_MULTIPLE)
+        selection.connect('changed', self.__select_changed)
+
+        self.set_drag_and_drop()
+
+        self.widget = gtk.VBox()
         scroll = gtk.ScrolledWindow()
-        scroll.add(self.widget)
+        scroll.add(self.treeview)
         scroll.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
         scroll.set_placement(gtk.CORNER_TOP_LEFT)
         viewport = gtk.Viewport()
         viewport.set_shadow_type(gtk.SHADOW_ETCHED_IN)
         viewport.add(scroll)
-        self.widget_tree = self.widget
+        self.widget.pack_start(viewport, expand=True, fill=True)
 
-        vbox.pack_start(viewport, expand=True, fill=True)
-
-        self.widget_tree.screen = screen
-
-        self.widget = vbox
-        self.reload = False
-        self.children = children
-
-        if children:
-            hbox = gtk.HBox()
-            self.widget.pack_start(hbox, expand=False, fill=False, padding=2)
-            keys = children.keys()
-            keys.sort()
-            for i in keys:
-                hbox2 = gtk.HBox()
-                hbox2.pack_start(children[i][1], expand=True, fill=False)
-                hbox2.pack_start(children[i][2], expand=True, fill=False)
-                hbox.pack_start(hbox2, expand=False, fill=False, padding=12)
-            hbox.show_all()
+        self.sum_box.show()
+        self.widget.pack_start(self.sum_box, expand=False, fill=False)
 
         self.display()
 
-        self.widget_tree.connect('button-press-event', self.__button_press)
-        self.widget_tree.connect_after('row-activated', self.__sig_switch)
-        if hasattr(self.widget_tree, 'set_rubber_banding'):
-            self.widget_tree.set_rubber_banding(True)
-        selection = self.widget_tree.get_selection()
-        selection.set_mode(gtk.SELECTION_MULTIPLE)
-        selection.connect('changed', self.__select_changed)
+    def parse(self, xml):
+        for node in xml.childNodes:
+            if node.nodeType != node.ELEMENT_NODE:
+                continue
+            if node.tagName == 'field':
+                self._parse_field(node)
+            elif node.tagName == 'button':
+                self._parse_button(node)
 
+        self.add_last_column()
+
+    def _parse_field(self, node):
+        group = self.screen.group
+        node_attrs = node_attributes(node)
+        name = node_attrs['name']
+        field = group.fields[name]
+        for b_field in ('readonly', 'expand', 'tree_invisible'):
+            if b_field in node_attrs:
+                node_attrs[b_field] = bool(int(node_attrs[b_field]))
+        for i_field in ('width', 'height'):
+            if i_field in node_attrs:
+                node_attrs[i_field] = int(node_attrs[i_field])
+        if 'widget' not in node_attrs:
+            node_attrs['widget'] = field.attrs['type']
+
+        for attr in ('relation', 'domain', 'selection',
+                'relation_field', 'string', 'views', 'invisible',
+                'add_remove', 'sort', 'context', 'size', 'filename',
+                'autocomplete', 'translate', 'create', 'delete',
+                'selection_change_with', 'schema_model'):
+            if (attr in field.attrs
+                    and attr not in node_attrs):
+                node_attrs[attr] = field.attrs[attr]
+
+        Widget = self.get_widget(node_attrs['widget'])
+        widget = Widget(self, node_attrs)
+        assert name not in self.widgets
+        self.widgets[name] = widget
+
+        column = gtk.TreeViewColumn(field.attrs['string'])
+        column._type = 'field'
+        column.name = name
+
+        prefixes = []
+        suffixes = []
+        if node_attrs['widget'] in ('url', 'email', 'callto', 'sip'):
+            prefixes.append(Affix(self, node_attrs,
+                    protocol=node_attrs['widget']))
+        if 'icon' in node_attrs:
+            prefixes.append(Affix(self, node_attrs))
+        for affix in node.childNodes:
+            affix_attrs = node_attributes(affix)
+            if 'name' not in affix_attrs:
+                affix_attrs['name'] = name
+            if affix.tagName == 'prefix':
+                list_ = prefixes
+            else:
+                list_ = suffixes
+            list_.append(Affix(self, affix_attrs))
+
+        for prefix in prefixes:
+            column.pack_start(prefix.renderer, expand=False)
+            column.set_cell_data_func(prefix.renderer,
+                prefix.setter)
+
+        column.pack_start(widget.renderer, expand=True)
+        column.set_cell_data_func(widget.renderer, widget.setter)
+
+        for suffix in suffixes:
+            column.pack_start(suffix.renderer, expand=False)
+            column.set_cell_data_func(suffix.renderer,
+                suffix.setter)
+
+        self.set_column_widget(column, field, node_attrs)
+        self.set_column_width(column, field, node_attrs)
+
+        if (not self.attributes.get('sequence')
+                and not self.children_field
+                and field.attrs.get('sortable', True)):
+            column.connect('clicked', self.sort_model)
+        column.set_visible(not node_attrs.get('tree_invisible', False))
+        if name == self.screen.exclude_field:
+            column.set_visible(False)
+
+        self.treeview.append_column(column)
+
+        self.add_sum(node_attrs)
+
+    def _parse_button(self, node):
+        node_attrs = node_attributes(node)
+        widget = Button(self, node_attrs)
+        self.state_widgets.append(widget)
+
+        column = gtk.TreeViewColumn(node_attrs.get('string', ''),
+            widget.renderer)
+        column._type = 'button'
+        column.name = None
+        column.set_cell_data_func(widget.renderer, widget.setter)
+
+        self.set_column_widget(column, None, node_attrs, arrow=False)
+        self.set_column_width(column, None, node_attrs)
+
+        column.set_visible(not node_attrs.get('tree_invisible', False))
+
+        self.treeview.append_column(column)
+
+    WIDGETS = {
+        'char': Char,
+        'many2one': M2O,
+        'date': Date,
+        'one2many': O2M,
+        'many2many': M2M,
+        'selection': Selection,
+        'float': Float,
+        'numeric': Float,
+        'float_time': FloatTime,
+        'integer': Int,
+        'biginteger': Int,
+        'datetime': Datetime,
+        'time': Time,
+        'boolean': Boolean,
+        'text': Char,
+        'url': URL,
+        'email': URL,
+        'callto': URL,
+        'sip': URL,
+        'progressbar': ProgressBar,
+        'reference': Reference,
+        'one2one': O2O,
+        'binary': Binary,
+        'image': Image,
+        }
+
+    @classmethod
+    def get_widget(cls, name):
+        return cls.WIDGETS[name]
+
+    @staticmethod
+    def set_column_widget(column, field, attributes, arrow=True):
+        tooltips = Tooltips()
+        hbox = gtk.HBox(False, 2)
+        label = gtk.Label(attributes['string'])
+        label.show()
+        help = attributes['string']
+        if field and field.attrs.get('help'):
+            help += '\n' + field.attrs['help']
+        elif attributes.get('help'):
+            help += '\n' + attributes['help']
+        tooltips.set_tip(label, help)
+        tooltips.enable()
+        arrow = gtk.Arrow(gtk.ARROW_DOWN, gtk.SHADOW_IN)
+        column.arrow = arrow
+        column.arrow_show = False
+        hbox.pack_start(label, True, True, 0)
+        if arrow:
+            hbox.pack_start(arrow, False, False, 0)
+            column.set_clickable(True)
+        hbox.show()
+        column.set_widget(hbox)
+
+    def set_column_width(self, column, field, attributes):
+        default_width = {
+            'integer': 60,
+            'biginteger': 60,
+            'float': 80,
+            'numeric': 80,
+            'float_time': 100,
+            'date': 110,
+            'datetime': 160,
+            'selection': 90,
+            'char': 100,
+            'one2many': 50,
+            'many2many': 50,
+            'boolean': 20,
+            'binary': 200,
+            }
+
+        width = self.screen.tree_column_width[self.screen.model_name].get(
+            column.name)
+        if not width:
+            if 'width' in attributes:
+                width = int(attributes['width'])
+            elif field:
+                width = default_width.get(field.attrs['type'], 100)
+            else:
+                width = 80
+        column.width = width
+        if width > 0:
+            column.set_fixed_width(width)
+        column.set_min_width(1)
+
+        expand = attributes.get('expand', False)
+        column.set_expand(expand)
+        column.set_resizable(True)
+        column.set_sizing(gtk.TREE_VIEW_COLUMN_FIXED)
+
+    def add_sum(self, attributes):
+        if 'sum' not in attributes:
+            return
+        if gtk.widget_get_default_direction() == gtk.TEXT_DIR_RTL:
+            text = _(':') + attributes['sum']
+        else:
+            text = attributes['sum'] + _(':')
+        label, sum_ = gtk.Label(text), gtk.Label()
+
+        hbox = gtk.HBox()
+        hbox.pack_start(label, expand=True, fill=False, padding=2)
+        hbox.pack_start(sum_, expand=True, fill=False, padding=2)
+        hbox.show_all()
+        self.sum_box.pack_start(hbox, expand=False, fill=False)
+
+        self.sum_widgets.append((attributes['name'], sum_))
+
+    def sort_model(self, column):
+        for col in self.treeview.get_columns():
+            if col != column:
+                col.arrow_show = False
+                col.arrow.hide()
+        self.screen.order = None
+        if not column.arrow_show:
+            column.arrow_show = True
+            column.arrow.set(gtk.ARROW_DOWN, gtk.SHADOW_IN)
+            column.arrow.show()
+            self.screen.order = [(column.name, 'ASC')]
+        else:
+            if column.arrow.get_property('arrow-type') == gtk.ARROW_DOWN:
+                column.arrow.set(gtk.ARROW_UP, gtk.SHADOW_IN)
+                self.screen.order = [(column.name, 'DESC')]
+            else:
+                column.arrow_show = False
+                column.arrow.hide()
+        model = self.treeview.get_model()
+        unsaved_records = [x for x in model.group if x.id < 0]
+        search_string = self.screen.screen_container.get_text() or None
+        if (self.screen.search_count == len(model)
+                or unsaved_records
+                or self.screen.parent):
+            ids = self.screen.search_filter(
+                search_string=search_string, only_ids=True)
+            model.sort(ids)
+        else:
+            self.screen.search_filter(search_string=search_string)
+
+    def add_last_column(self):
+        for column in self.treeview.get_columns():
+            if column.get_expand():
+                break
+        else:
+            column = gtk.TreeViewColumn()
+            column._type = 'fill'
+            column.name = None
+            column.arrow = gtk.Arrow(gtk.ARROW_DOWN, gtk.SHADOW_IN)
+            column.arrow_show = False
+            column.set_sizing(gtk.TREE_VIEW_COLUMN_FIXED)
+            self.treeview.append_column(column)
+
+    def set_drag_and_drop(self):
         dnd = False
         if self.children_field:
-            children_field = self.widget_tree.cells.get(self.children_field)
-            if children_field:
-                parent_name = children_field.attrs.get('relation_field')
-                dnd = parent_name in self.widget_tree.cells
-        elif self.widget_tree.sequence:
+            children = self.screen.group.fields.get(self.children_field)
+            if children:
+                parent_name = children.attrs.get('relation_field')
+                dnd = parent_name in self.widgets
+        elif self.attributes.get('sequence'):
             dnd = True
         # Disable DnD on mac until it is fully supported
         if sys.platform == 'darwin':
             dnd = False
-        if screen.readonly:
+        if self.screen.readonly:
             dnd = False
-        if dnd:
-            self.widget_tree.drag_source_set(
-                gtk.gdk.BUTTON1_MASK | gtk.gdk.BUTTON3_MASK,
-                [('MY_TREE_MODEL_ROW', gtk.TARGET_SAME_WIDGET, 0)],
-                gtk.gdk.ACTION_MOVE)
-            self.widget_tree.drag_dest_set(gtk.DEST_DEFAULT_ALL,
-                [('MY_TREE_MODEL_ROW', gtk.TARGET_SAME_WIDGET, 0)],
-                gtk.gdk.ACTION_MOVE)
 
-            self.widget_tree.connect('drag-begin', self.drag_begin)
-            self.widget_tree.connect('drag-motion', self.drag_motion)
-            self.widget_tree.connect('drag-drop', self.drag_drop)
-            self.widget_tree.connect("drag-data-get", self.drag_data_get)
-            self.widget_tree.connect('drag-data-received',
-                self.drag_data_received)
-            self.widget_tree.connect('drag-data-delete', self.drag_data_delete)
+        if not dnd:
+            return
 
-        self.widget_tree.connect('key_press_event', self.on_keypress)
-        if self.children_field:
-            self.widget_tree.connect('test-expand-row', self.test_expand_row)
-            self.widget_tree.set_expander_column(
-                self.widget_tree.get_column(0))
+        self.treeview.drag_source_set(
+            gtk.gdk.BUTTON1_MASK | gtk.gdk.BUTTON3_MASK,
+            [('MY_TREE_MODEL_ROW', gtk.TARGET_SAME_WIDGET, 0)],
+            gtk.gdk.ACTION_MOVE)
+        self.treeview.drag_dest_set(gtk.DEST_DEFAULT_ALL,
+            [('MY_TREE_MODEL_ROW', gtk.TARGET_SAME_WIDGET, 0)],
+            gtk.gdk.ACTION_MOVE)
+
+        self.treeview.connect('drag-begin', self.drag_begin)
+        self.treeview.connect('drag-motion', self.drag_motion)
+        self.treeview.connect('drag-drop', self.drag_drop)
+        self.treeview.connect("drag-data-get", self.drag_data_get)
+        self.treeview.connect('drag-data-received',
+            self.drag_data_received)
+        self.treeview.connect('drag-data-delete', self.drag_data_delete)
 
     @property
     def modified(self):
@@ -329,10 +604,10 @@ class ViewList(ParserView):
 
     @property
     def editable(self):
-        return bool(getattr(self.widget_tree, 'editable', False))
+        return bool(getattr(self.treeview, 'editable', False))
 
     def get_fields(self):
-        return [col.name for col in self.widget_tree.get_columns() if col.name]
+        return [col.name for col in self.treeview.get_columns() if col.name]
 
     def get_buttons(self):
         return [b for b in self.state_widgets
@@ -390,7 +665,7 @@ class ViewList(ParserView):
         iter_ = model.iter_children(iter_)
         if not iter_:
             return False
-        fields = [col.name for col in self.widget_tree.get_columns()
+        fields = [col.name for col in self.treeview.get_columns()
                 if col.name]
         while iter_:
             record = model.get_value(iter_, 0)
@@ -405,14 +680,14 @@ class ViewList(ParserView):
     def on_copy(self):
         for clipboard_type in (gtk.gdk.SELECTION_CLIPBOARD,
                 gtk.gdk.SELECTION_PRIMARY):
-            clipboard = self.widget_tree.get_clipboard(clipboard_type)
+            clipboard = self.treeview.get_clipboard(clipboard_type)
             targets = [
                 ('STRING', 0, 0),
                 ('TEXT', 0, 1),
                 ('COMPOUND_TEXT', 0, 2),
                 ('UTF8_STRING', 0, 3)
             ]
-            selection = self.widget_tree.get_selection()
+            selection = self.treeview.get_selection()
             # Set to clipboard directly if not too much selected rows
             # to speed up paste
             # Don't use set_with_data on mac see:
@@ -429,11 +704,13 @@ class ViewList(ParserView):
     def copy_foreach(self, treemodel, path, iter, data):
         record = treemodel.get_value(iter, 0)
         values = []
-        for col in self.widget_tree.get_columns():
+        for col in self.treeview.get_columns():
             if not col.get_visible() or not col.name:
                 continue
-            cell = self.widget_tree.cells[col.name]
-            values.append('"' + str(cell.get_textual_value(record)) + '"')
+            widget = self.widgets[col.name]
+            values.append('"'
+                + str(widget.get_textual_value(record)).replace('"', '""')
+                + '"')
         data.append('\t'.join(values))
         return
 
@@ -459,15 +736,15 @@ class ViewList(ParserView):
         data = []
         for clipboard_type in (gtk.gdk.SELECTION_CLIPBOARD,
                 gtk.gdk.SELECTION_PRIMARY):
-            clipboard = self.widget_tree.get_clipboard(clipboard_type)
+            clipboard = self.treeview.get_clipboard(clipboard_type)
             text = clipboard.wait_for_text()
             if not text:
                 continue
             data = [[unquote(v) for v in l.split('\t')]
                 for l in text.splitlines()]
             break
-        col = self.widget_tree.get_cursor()[1]
-        columns = [c for c in self.widget_tree.get_columns()
+        col = self.treeview.get_cursor()[1]
+        columns = [c for c in self.treeview.get_columns()
             if c.get_visible() and c.name]
         if col in columns:
             idx = columns.index(col)
@@ -489,10 +766,10 @@ class ViewList(ParserView):
                 group.add(record)
             record = group[idx]
             for col, value in zip(columns, line):
-                cell = self.widget_tree.cells[col.name]
-                if cell.get_textual_value(record) != value:
-                    cell.value_from_text(record, value)
-                    if value and not cell.get_textual_value(record):
+                widget = self.widgets[col.name]
+                if widget.get_textual_value(record) != value:
+                    widget.value_from_text(record, value)
+                    if value and not widget.get_textual_value(record):
                         # Stop setting value if a value is correctly set
                         idx = len(group)
                         break
@@ -527,9 +804,9 @@ class ViewList(ParserView):
             etime):
         treeview.emit_stop_by_name('drag-data-get')
 
-        def _func_sel_get(store, path, iter_, data):
-            value = store.get_value(iter_, 0)
-            data.append(json.dumps(value.get_path(store.group)))
+        def _func_sel_get(model, path, iter_, data):
+            value = model.get_value(iter_, 0)
+            data.append(json.dumps(value.get_path(model.group)))
         data = []
         treeselection = treeview.get_selection()
         treeselection.selected_foreach(_func_sel_get, data)
@@ -542,8 +819,8 @@ class ViewList(ParserView):
     def drag_data_received(self, treeview, context, x, y, selection,
             info, etime):
         treeview.emit_stop_by_name('drag-data-received')
-        if treeview.sequence:
-            field = self.screen.group.fields[treeview.sequence]
+        if self.attributes.get('sequence'):
+            field = self.screen.group.fields[self.attributes['sequence']]
             for record in self.screen.group:
                 if field.get_state_attrs(
                         record).get('readonly', False):
@@ -559,13 +836,13 @@ class ViewList(ParserView):
                 if renderer.props.editing:
                     return
 
-        store = treeview.get_model()
+        model = treeview.get_model()
         try:
             data = json.loads(selection.data)
         except ValueError:
             return
-        record = store.group.get_by_path(data)
-        record_path = store.on_get_path(record)
+        record = model.group.get_by_path(data)
+        record_path = model.on_get_path(record)
         drop_info = treeview.get_dest_row_at_pos(x, y)
 
         def check_recursion(from_, to):
@@ -586,16 +863,16 @@ class ViewList(ParserView):
             if not check_recursion(record_path, check_path):
                 return
             if position == gtk.TREE_VIEW_DROP_BEFORE:
-                store.move_before(record, path)
+                model.move_before(record, path)
             elif position == gtk.TREE_VIEW_DROP_AFTER:
-                store.move_after(record, path)
+                model.move_after(record, path)
             elif self.children_field:
-                store.move_into(record, path)
+                model.move_into(record, path)
         else:
-            store.move_after(record, (len(store) - 1,))
+            model.move_after(record, (len(model) - 1,))
         context.drop_finish(False, etime)
-        if treeview.sequence:
-            record.group.set_sequence(field=treeview.sequence)
+        if self.attributes.get('sequence'):
+            record.group.set_sequence(field=self.attributes['sequence'])
         return True
 
     def drag_data_delete(self, treeview, context):
@@ -634,7 +911,7 @@ class ViewList(ParserView):
                     parent = parent.parent
                 else:
                     populate(menu, group.model_name, record)
-                for col in self.widget_tree.get_columns():
+                for col in self.treeview.get_columns():
                     if not col.get_visible() or not col.name:
                         continue
                     field = group.fields[col.name]
@@ -662,11 +939,12 @@ class ViewList(ParserView):
         return False
 
     def group_list_changed(self, group, signal):
-        if self.store is not None:
+        model = self.treeview.get_model()
+        if model is not None:
             if signal[0] == 'record-added':
-                self.store.added(group, signal[1])
+                model.added(group, signal[1])
             elif signal[0] == 'record-removed':
-                self.store.removed(group, signal[1])
+                model.removed(group, signal[1])
         self.display()
 
     def __str__(self):
@@ -680,7 +958,7 @@ class ViewList(ParserView):
             return
         fields = {}
         last_col = None
-        for col in self.widget_tree.get_columns():
+        for col in self.treeview.get_columns():
             if col.get_visible():
                 last_col = col
             if not hasattr(col, 'name') or not hasattr(col, 'width'):
@@ -704,7 +982,7 @@ class ViewList(ParserView):
 
     def destroy(self):
         self.save_width_height()
-        self.widget_tree.destroy()
+        self.treeview.destroy()
 
     def __sig_switch(self, treeview, path, column):
         if column._type == 'button':
@@ -768,40 +1046,39 @@ class ViewList(ParserView):
                             go_previous()
                 # Delay the pre_validate to let GTK process the current event
                 gobject.idle_add(pre_validate)
-        self.update_children()
+        self.update_sum()
 
     def set_value(self):
         if self.editable:
-            self.widget_tree.set_value()
+            self.treeview.set_value()
 
     def reset(self):
         pass
 
-    # self.widget.set_model(self.store) could be removed if the store
-    # has not changed -> better ergonomy. To test
     def display(self):
+        self.treeview.display_counter += 1
         current_record = self.screen.current_record
         if (self.reload
-                or not self.widget_tree.get_model()
+                or not self.treeview.get_model()
                 or (self.screen.group !=
-                    self.widget_tree.get_model().group)):
-            self.store = AdaptModelGroup(self.screen.group,
+                    self.treeview.get_model().group)):
+            model = AdaptModelGroup(self.screen.group,
                     self.children_field)
-            self.widget_tree.set_model(self.store)
+            self.treeview.set_model(model)
             # __select_changed resets current_record to None
             self.screen.current_record = current_record
             if current_record:
-                selection = self.widget_tree.get_selection()
-                path = self.store.on_get_path(current_record)
+                selection = self.treeview.get_selection()
+                path = model.on_get_path(current_record)
                 selection.select_path(path)
         self.reload = False
         if not current_record:
-            selection = self.widget_tree.get_selection()
+            selection = self.treeview.get_selection()
             selection.unselect_all()
-        self.widget_tree.queue_draw()
+        self.treeview.queue_draw()
         if self.editable:
             self.set_state()
-        self.update_children()
+        self.update_sum()
 
     def set_state(self):
         record = self.screen.current_record
@@ -811,62 +1088,64 @@ class ViewList(ParserView):
                 if field:
                     field.state_set(record)
 
-    def update_children(self):
+    @delay
+    def update_sum(self):
         selected_records = self.selected_records
-        for child in self.children:
-            value = 0
-            value_selected = 0
+        for name, label in self.sum_widgets:
+            sum_ = 0
+            selected_sum = 0
             loaded = True
-            child_fieldname = self.children[child][0]
+            digit = 0
             for record in self.screen.group:
-                if not record.get_loaded([child_fieldname]):
+                if not record.get_loaded([name]) and record.id >= 0:
                     loaded = False
                     break
-                field_value = record.fields_get()[child_fieldname].get(record)
-                if field_value is not None:
-                    value += field_value
+                field = record[name]
+                value = field.get(record)
+                if value is not None:
+                    sum_ += value
                     if record in selected_records or not selected_records:
-                        value_selected += field_value
+                        selected_sum += value
+                    digit = max(field.digits(record)[1], digit)
 
             if loaded:
-                label_str = locale.format('%.*f',
-                    (self.children[child][3], value_selected), True)
-                label_str += ' / '
-                label_str += locale.format('%.*f',
-                    (self.children[child][3], value), True)
+                text = '%s / %s' % (
+                    locale.format('%.*f', (digit, selected_sum), True),
+                    locale.format('%.*f', (digit, sum_), True))
             else:
-                label_str = '-'
-            self.children[child][2].set_text(label_str)
+                text = '-'
+            label.set_text(text)
 
     def set_cursor(self, new=False, reset_view=True):
-        self.widget_tree.grab_focus()
+        self.treeview.grab_focus()
         if self.screen.current_record:
-            path = self.store.on_get_path(self.screen.current_record)
-            if self.store.get_flags() & gtk.TREE_MODEL_LIST_ONLY:
+            model = self.treeview.get_model()
+            path = model.on_get_path(self.screen.current_record)
+            if model.get_flags() & gtk.TREE_MODEL_LIST_ONLY:
                 path = (path[0],)
-            focus_column = self.widget_tree.next_column(path, editable=new)
+            focus_column = self.treeview.next_column(path, editable=new)
             if path[:-1]:
-                self.widget_tree.expand_to_path(path[:-1])
-            self.widget_tree.scroll_to_cell(path, focus_column,
+                self.treeview.expand_to_path(path[:-1])
+            self.treeview.scroll_to_cell(path, focus_column,
                 use_align=False)
-            current_path = self.widget_tree.get_cursor()[0]
+            current_path = self.treeview.get_cursor()[0]
             selected_path = \
-                self.widget_tree.get_selection().get_selected_rows()[1]
+                self.treeview.get_selection().get_selected_rows()[1]
             if (current_path != path and path not in selected_path) or new:
-                self.widget_tree.set_cursor(path, focus_column, new)
+                self.treeview.set_cursor(path, focus_column, new)
 
     @property
     def selected_records(self):
-        def _func_sel_get(store, path, iter, records):
-            records.append(store.on_get_iter(path))
+        def _func_sel_get(model, path, iter, records):
+            records.append(model.on_get_iter(path))
         records = []
-        sel = self.widget_tree.get_selection()
+        sel = self.treeview.get_selection()
         sel.selected_foreach(_func_sel_get, records)
         return records
 
     def unset_editable(self):
-        self.widget_tree.editable = False
-        for col in self.widget_tree.get_columns():
+        self.treeview.editable = False
+        for col in self.treeview.get_columns():
             for renderer in col.get_cell_renderers():
                 if isinstance(renderer, CellRendererToggle):
                     renderer.set_property('activatable', False)
@@ -878,7 +1157,7 @@ class ViewList(ParserView):
                     renderer.set_property('editable', False)
 
     def get_selected_paths(self):
-        selection = self.widget_tree.get_selection()
+        selection = self.treeview.get_selection()
         model, rows = selection.get_selected_rows()
         id_paths = []
         for row in rows:
@@ -891,17 +1170,17 @@ class ViewList(ParserView):
         return id_paths
 
     def select_nodes(self, nodes):
-        selection = self.widget_tree.get_selection()
+        selection = self.treeview.get_selection()
         if not nodes:
             return
         selection.unselect_all()
         scroll = False
         for node in nodes:
-            path = path_convert_id2pos(self.store, node)
+            path = path_convert_id2pos(self.treeview.get_model(), node)
             if path:
                 selection.select_path(path)
                 if not scroll:
-                    self.widget_tree.scroll_to_cell(path)
+                    self.treeview.scroll_to_cell(path)
                     scroll = True
 
     def get_expanded_paths(self, starting_path=None, starting_id_path=None):
@@ -912,12 +1191,13 @@ class ViewList(ParserView):
         if not starting_id_path:
             starting_id_path = []
         id_paths = []
-        record = self.store.on_get_iter(starting_path)
-        for path_idx in range(self.store.on_iter_n_children(record)):
+        model = self.treeview.get_model()
+        record = model.on_get_iter(starting_path)
+        for path_idx in range(model.on_iter_n_children(record)):
             path = starting_path + [path_idx]
-            expanded = self.widget_tree.row_expanded(tuple(path))
+            expanded = self.treeview.row_expanded(tuple(path))
             if expanded:
-                expanded_record = self.store.on_get_iter(path)
+                expanded_record = model.on_get_iter(path)
                 id_path = starting_id_path + [expanded_record.id]
                 id_paths.append(id_path)
                 child_id_paths = self.get_expanded_paths(path, id_path)
@@ -925,7 +1205,8 @@ class ViewList(ParserView):
         return id_paths
 
     def expand_nodes(self, nodes):
+        model = self.treeview.get_model()
         for node in nodes:
-            expand_path = path_convert_id2pos(self.store, node)
+            expand_path = path_convert_id2pos(model, node)
             if expand_path:
-                self.widget_tree.expand_to_path(expand_path)
+                self.treeview.expand_to_path(expand_path)
