@@ -110,7 +110,7 @@ class Wizard(object):
                     Action._exec_action(*action, context=self.context.copy())
 
             if self.state == self.end_state:
-                self.end(execute_actions)
+                self.end(lambda *a: execute_actions())
             else:
                 execute_actions()
             self.__processing = False
@@ -150,8 +150,10 @@ class Wizard(object):
         button.show()
         return button
 
-    def _record_modified(self, screen, signal):
-        record = screen.current_record
+    def _record_modified(self, screen, record):
+        self.update_buttons(record)
+
+    def update_buttons(self, record):
         for button in self.states.itervalues():
             button.state_set(record)
 
@@ -223,6 +225,7 @@ class Wizard(object):
 
         self.screen.new(default=False)
         self.screen.current_record.set_default(defaults)
+        self.update_buttons(self.screen.current_record)
         self.screen.set_cursor()
 
 
@@ -341,17 +344,20 @@ class WizardDialog(Wizard, NoModal):
             dialog = self.page.dialogs[-1]
         else:
             dialog = self.page
-        if (getattr(dialog, 'screen', None)
-                and dialog.screen.current_record
-                and self.sensible_widget != main.window):
-            if dialog.screen.model_name == self.model:
-                ids = self.ids
-            else:
-                # Wizard run from a children record so reload parent record
-                ids = [dialog.screen.current_record.id]
-            dialog.screen.reload(ids, written=True)
+        screen = getattr(dialog, 'screen', None)
+        if self.sensible_widget == main.window:
+            screen = main.menu_screen
+        if screen:
+            if (screen.current_record
+                    and self.sensible_widget != main.window):
+                if screen.model_name == self.model:
+                    ids = self.ids
+                else:
+                    # Wizard run from a children record so reload parent record
+                    ids = [screen.current_record.id]
+                screen.reload(ids, written=True)
             if action:
-                dialog.screen.client_action(action)
+                screen.client_action(action)
 
     def end(self, callback=None):
         def end_callback(action):
